class Python(object):
  '''
  This class is intended to encapsulate operations which are generic to the Python language (both Python itself and C extensions)
  '''
  
  def __init__(self, IOR = None):
    if IOR is None:
      import ASE.Compiler.Python.IOR
      self.IOR = ASE.Compiler.Python.IOR.IOR()
    else:
      self.IOR = IOR
    self.Cxx = self.IOR.Cxx
    self.SIDL = self.IOR.SIDL
    self.sidlObjTypes = {}

    self.wrapIOR = {}

    self.extractIOR = {}

    self.wrapArray = {}

    self.extractArray = {}

    self.extractIterator = {}

    return

  def clear(self):
    self.IOR.clear()
    del self.typeMap
    del self.typeCodes
    del self.refTypeType
    del self.objType
    del self.arrayType
    del self.typeType
    self.sidlObjTypes.clear()
    self.wrapIOR.clear()
    self.extractIOR.clear()
    self.wrapArray.clear()
    self.extractArray.clear()
    return

  def getTypeMap(self):
    if not hasattr(self, '_typeMap'):
      self._typeMap = self.createTypeMap()
    return self._typeMap
  def setTypeMap(self, typeMap):
    self._typeMap = typeMap
    return
  def delTypeMap(self):
    if hasattr(self, '_typeMap'):
      del self._typeMap
    return
  typeMap = property(getTypeMap, setTypeMap, delTypeMap, doc = 'A map from typenames to Python C extension type objects')

  def getTypeCodes(self):
    if not hasattr(self, '_typeCodes'):
      self._typeCodes = self.createTypeCodes()
    return self._typeCodes
  def setTypeCodes(self, typeCodes):
    self._typeCodes = typeCodes
    return
  def delTypeCodes(self):
    if hasattr(self, '_typeCodes'):
      del self._typeCodes
    return
  typeCodes = property(getTypeCodes, setTypeCodes, delTypeCodes, doc = 'A map from typenames to Numeric typecode variables')
  
  def createTypeMap(self):
    '''
    Create a map from SIDL base types to C extension types
    '''
    import ASE.Compiler.Cxx.Type
    typeMap = {}

    typeMap['bool'] = self.IOR.typeMap['bool']
    typeMap['char'] = self.IOR.typeMap['char']
    pyType = ASE.Compiler.Cxx.Type.Type()
    pyType.setIdentifier('Py_complex')
    typeMap['dcomplex'] = pyType
    typeMap['double'] = self.IOR.typeMap['double']
    typeMap['fcomplex'] = typeMap['dcomplex']
    typeMap['float'] = self.IOR.typeMap['float']
    pyType = ASE.Compiler.Cxx.Type.Type()
    pyType.setIdentifier('long')
    pyType.setBaseType(1)
    typeMap['int'] = pyType
    pyType = ASE.Compiler.Cxx.Type.Type()
    pyType.setIdentifier('long long')
    pyType.setBaseType(1)
    typeMap['long'] = pyType
    typeMap['void'] = self.IOR.typeMap['void']
    typeMap['opaque'] = self.IOR.typeMap['opaque']
    typeMap['string'] = self.IOR.typeMap['string']
    return typeMap

  def getString(self, s):
    import ASE.Compiler.Python.Const
    string = ASE.Compiler.Python.Const.Const()
    string.setValue_string('\'' + s + '\'')
    return string
  
  def getType(self, type):
    '''
    Return the Python C extension type corresponding to the given SIDL type
    '''
    if type.getBaseType():
      try:
        pyType = self.typeMap[type.getIdentifier()]
      except KeyError:
        self.raiseException('No base type: ' + str(type.getIdentifier()))
    elif type.isInstanceOf('ASE.Compiler.SIDL.Enumeration'):
      pyType = self.typeMap['int']
    elif type.isInstanceOf('ASE.Compiler.SIDL.Array'):
      pyType = self.IOR.getType(type)
    elif type.isInstanceOf('ASE.Compiler.SIDL.Iterator'):
      pyType = self.IOR.getType(type)
    elif type.isInstanceOf('ASE.Compiler.SIDL.Interface'):
      pyType = self.IOR.getType(type)
    else:
      self.raiseException('Unable to map type: ' + str(type.getFullIdentifier()))
    return pyType
  
  def createTypeCodes(self):
    '''
    Create a map from SIDL base types to Numeric typecodes
    '''
    typeCodes = {}

    typeCodes['char'] = self.Cxx.getVar('PyArray_CHAR')
    typeCodes['dcomplex'] = self.Cxx.getVar('PyArray_CDOUBLE')
    typeCodes['double'] = self.Cxx.getVar('PyArray_DOUBLE')
    typeCodes['fcomplex'] = self.Cxx.getVar('PyArray_CFLOAT')
    typeCodes['float'] = self.Cxx.getVar('PyArray_FLOAT')
    typeCodes['int'] = self.Cxx.getVar('PyArray_INT')
    typeCodes['bool'] = typeCodes['int']
    typeCodes['long'] = self.Cxx.getVar('PyArray_LONG')
    typeCodes['object'] = self.Cxx.getVar('PyArray_OBJECT')
    typeCodes['opaque'] = typeCodes['object']
    typeCodes['string'] = typeCodes['object']
    typeCodes['void'] = self.Cxx.getVar('PyArray_NOTYPE')
    return typeCodes
  
  def getNumericTypeCode(self, type):
    '''
    Return the Numeric typecode corresponding to the given SIDL type
    '''
    if type.getBaseType():
      try:
        typeCode = self.typeCodes[type.getIdentifier()]
      except KeyError:
        self.raiseException('No base type: ' + str(type.getIdentifier()))
    elif type.isInstanceOf('ASE.Compiler.SIDL.Enumeration'):
      typeCode = self.typeCodes['int']
    elif type.isInstanceOf('ASE.Compiler.SIDL.Array'):
      self.raiseException('Nested arrays not allowed in SIDL')
    elif type.isInstanceOf('ASE.Compiler.SIDL.Interface'):
      typeCode = self.typeCodes['object']
    else:
      typeCode = self.typeCodes['void']
    return typeCode
  
  def getRefTypeType(self):
    if not hasattr(self, '_refTypeType'):
      import ASE.Compiler.Cxx.Enumeration
      self._refTypeType = ASE.Compiler.Cxx.Enumeration.Enumeration()
      self._refTypeType.setIdentifier('ReferenceType')
    return self._refTypeType
  def setRefTypeType(self, refTypeType):
    self._refTypeType = refTypeType
    return
  def delRefTypeType(self):
    if hasattr(self, '_refTypeType'):
      del self._refTypeType
    return
  refTypeType = property(getRefTypeType, setRefTypeType, delRefTypeType, doc = 'The Python C extension type used to indicate the type of Python reference')
  
  def getObjType(self):
    if not hasattr(self, '_objType'):
      import ASE.Compiler.Cxx.Pointer
      import ASE.Compiler.Cxx.Type
      objStruct = ASE.Compiler.Cxx.Type.Type()
      objStruct.setIdentifier('PyObject')
      self._objType = ASE.Compiler.Cxx.Pointer.Pointer()
      self._objType.setChildren([objStruct])
    return self._objType
  def setObjType(self, objType):
    self._objType = objType
    return
  def delObjType(self):
    if hasattr(self, '_objType'):
      del self._objType
    return
  objType = property(getObjType, setObjType, delObjType, doc = 'The Python C extension type for Python instance objects')
  
  def getArrayType(self):
    if not hasattr(self, '_arrayType'):
      import ASE.Compiler.Cxx.Pointer
      import ASE.Compiler.Cxx.Type
      objStruct = ASE.Compiler.Cxx.Type.Type()
      objStruct.setIdentifier('PyArrayObject')
      self._arrayType = ASE.Compiler.Cxx.Pointer.Pointer()
      self._arrayType.setChildren([objStruct])
    return self._arrayType
  def setArrayType(self, arrayType):
    self._arrayType = arrayType
    return
  def delArrayType(self):
    if hasattr(self, '_arrayType'):
      del self._arrayType
    return
  arrayType = property(getArrayType, setArrayType, delArrayType, doc = 'The Python C extension type for Numeric array objects')
  
  def getTypeType(self):
    if not hasattr(self, '_typeType'):
      import ASE.Compiler.Cxx.Pointer
      import ASE.Compiler.Cxx.Type
      objStruct = ASE.Compiler.Cxx.Type.Type()
      objStruct.setIdentifier('PyTypeObject')
      self._typeType = ASE.Compiler.Cxx.Pointer.Pointer()
      self._typeType.setChildren([objStruct])
    return self._typeType
  def setTypeType(self, typeType):
    self._typeType = typeType
    return
  def delTypeType(self):
    if hasattr(self, '_typeType'):
      del self._typeType
    return
  typeType = property(getTypeType, setTypeType, delTypeType, doc = 'The Python C extension type for Python type objects')
  
  def getSIDLObjectType(self, interface):
    '''
    Return the specific Python C extension type for Python instance objects for the given SIDL interface
    '''
    if not interface in self.sidlObjTypes:
      import ASE.Compiler.Cxx.Pointer
      import ASE.Compiler.Cxx.Struct
      objStruct = ASE.Compiler.Cxx.Struct.Struct()
      objStruct.setIdentifier(interface.getFullIdentifier().replace('.', '_') + '_PythonObject')
      self.sidlObjTypes[interface] = ASE.Compiler.Cxx.Pointer.Pointer()
      self.sidlObjTypes[interface].setChildren([objStruct])
    return self.sidlObjTypes[interface]
  
  def getRefTypeEnumeration(self):
    '''
    Return the Python C extension enumeration declaration for reference type
    '''
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Enumeration
    import ASE.Compiler.Cxx.Enumerator
    stolenRef = ASE.Compiler.Cxx.Enumerator.Enumerator()
    stolenRef.setIdentifier('STOLEN_REF')
    weakRef = ASE.Compiler.Cxx.Enumerator.Enumerator()
    weakRef.setIdentifier('WEAK_REF')
    newRef = ASE.Compiler.Cxx.Enumerator.Enumerator()
    newRef.setIdentifier('NEW_REF')
    refType = ASE.Compiler.Cxx.Enumeration.Enumeration()
    refType.setIdentifier('ReferenceType')
    refType.setChildren([stolenRef, weakRef, newRef])
    refDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    refDecl.setType(refType)
    return self.Cxx.getDecl(refDecl)
  
  def getSIDLObject(self, klass):
    '''
    Return the Python C extension structure declaration for the given class
    '''
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Enumeration
    import ASE.Compiler.Cxx.Macro
    import ASE.Compiler.Cxx.Struct
    head = ASE.Compiler.Cxx.Macro.Macro()
    head.setIdentifier('PyObject_HEAD')
    objIOR = ASE.Compiler.Cxx.Declarator.Declarator()
    objIOR.setIdentifier('ior')
    objIOR.setType(self.IOR.getType(klass))
    refType = ASE.Compiler.Cxx.Enumeration.Enumeration()
    refType.setIdentifier('ReferenceType')
    objRef = ASE.Compiler.Cxx.Declarator.Declarator()
    objRef.setIdentifier('refType')
    objRef.setType(refType)
    weakRefList = ASE.Compiler.Cxx.Declarator.Declarator()
    weakRefList.setIdentifier('weakRefList')
    weakRefList.setType(self.objType)
    objStruct = ASE.Compiler.Cxx.Struct.Struct()
    objStruct.setIdentifier(klass.getFullIdentifier().replace('.', '_') + '_PythonObject')
    objStruct.setChildren([self.Cxx.getExpStmt(head), self.Cxx.getDecl(objIOR), self.Cxx.getDecl(objRef), self.Cxx.getDecl(weakRefList)])
    objDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    objDecl.setType(objStruct)
    return self.Cxx.getDecl(objDecl)
  
  def getClearError(self):
    '''
    Return the AST which clears any Python error which has been set
       - Should probably be replaced by a PyErr_Fetch()
    '''
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.If
    clearError = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    clearError.setChildren([self.Cxx.getVar('PyErr_Clear')])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([self.Cxx.getExpStmt(clearError)])
    pythonError = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    pythonError.setChildren([self.Cxx.getVar('PyErr_Occurred')])
    errorCheck = ASE.Compiler.Cxx.If.If()
    errorCheck.setBranch(pythonError)
    errorCheck.setChildren([compound])
    return errorCheck
  
  def getSuspendError(self):
    '''
    Return the AST which suspends any Python error which has been set
    '''
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.If
    suspendError = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    suspendError.setChildren([self.Cxx.getVar('PyErr_Fetch')])
    suspendError.setArguments([self.Cxx.getAddress(self.Cxx.getVar('_saveExcType')), self.Cxx.getAddress(self.Cxx.getVar('_saveExcValue')), self.Cxx.getAddress(self.Cxx.getVar('_saveExcTraceback'))])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([self.Cxx.getExpStmt(suspendError)])
    pythonError = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    pythonError.setChildren([self.Cxx.getVar('PyErr_Occurred')])
    errorCheck = ASE.Compiler.Cxx.If.If()
    errorCheck.setBranch(pythonError)
    errorCheck.setChildren([compound])
    return errorCheck
  
  def getRestoreError(self):
    '''
    Return the AST which restores any Python error which has been set
    '''
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.If
    suspendError = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    suspendError.setChildren([self.Cxx.getVar('PyErr_Restore')])
    suspendError.setArguments([self.Cxx.getVar('_saveExcType'), self.Cxx.getVar('_saveExcValue'), self.Cxx.getVar('_saveExcTraceback')])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([self.Cxx.getExpStmt(suspendError)])
    errorCheck = ASE.Compiler.Cxx.If.If()
    errorCheck.setBranch(self.Cxx.getVar('_saveExcType'))
    errorCheck.setChildren([compound])
    return errorCheck
  
  def getErrorCheck(self, exceptionType, errorMsg, errorVar = None, returnVar = None, branchExp = None):
    '''
    Return the AST for the check for a Python exception
       - exceptionType is the type of exception to be raised on an error, e.g. PyExc_ImportError
       - errorMsg is the string for the raised exception
    '''
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.Return
    if not errorVar is None:
      import ASE.Compiler.Cxx.Negation
      pythonError = ASE.Compiler.Cxx.Negation.Negation()
      pythonError.setChildren([errorVar])
    elif not branchExp is None:
      pythonError = branchExp
    else:
      pythonError = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      pythonError.setChildren([self.Cxx.getVar('PyErr_Occurred')])
    exception = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    exception.setChildren([self.Cxx.getVar('PyErr_SetString')])
    exception.setArguments([self.Cxx.getVar(exceptionType), self.Cxx.getString(errorMsg)])
    returnVal = ASE.Compiler.Cxx.Return.Return()
    if not returnVar is None:
      returnVal.setChildren([returnVar])
    handleError = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    handleError.setChildren([self.Cxx.getExpStmt(exception), returnVal])
    errorCheck = ASE.Compiler.Cxx.If.If()
    errorCheck.setBranch(pythonError)
    errorCheck.setChildren([handleError])
    return errorCheck
  
  def escapeComment(self, line):
    return line.replace('\\', '\\\\').replace('"', '\\"').replace('\'', '\\\'')
  
  def getModeName(self, mode):
    import ASE.Compiler.ParameterAccessMode
    if mode == ASE.Compiler.ParameterAccessMode.IN:
      return 'in'
    elif mode == ASE.Compiler.ParameterAccessMode.INOUT:
      return 'inout'
    elif mode == ASE.Compiler.ParameterAccessMode.OUT:
      return 'out'
    elif mode == ASE.Compiler.ParameterAccessMode.RETURN:
      return 'return'
    raise RuntimeError('Invalid parameter access mode: ' + str(mode))
  
  def getParameterString(self, parameter):
    name = parameter.getIdentifier()
    if not name:
      name = '_result'
    return self.getModeName(parameter.getAccessMode()) + ' ' + parameter.getType().getFullIdentifier() + ' ' + name
  
  def getObjectComment(self, obj):
    return ''.join(['\\n\\\n'.join(map(self.escapeComment, comment.getComment().split('\n'))) for comment in obj.getComments()])
  
  def getMethodComment(self, method):
    import ASE.Compiler.SIDL.Parameter
    outParams = method.getOutParameters()
    end = '\\n\\\n'
    str = method.getIdentifier() + '('
    num = len(method.getChildren())
    for (c, child) in enumerate(method.getChildren()):
      str += self.getParameterString(ASE.Compiler.SIDL.Parameter.Parameter(child))
      if c < num - 1:
        str += ', '
    str += ')' + end
    str += 'RETURNS '
    num = len(outParams)
    if num:
      str += '('
      for (p, parameter) in enumerate(outParams):
        str += self.getParameterString(parameter)
        if p < num - 1:
          str += ', '
      str += ')'
    else:
      str += 'None' + end
    str += end
    str += self.getObjectComment(method)
    return str
  
  def getNew(self, interface):
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.Initializer
    import ASE.Compiler.Cxx.Parameter
    import ASE.Compiler.Cxx.Return
    body = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    selfVar = self.Cxx.getVar('self')
    typeVar = self.Cxx.getVar('type')
    iorVar = self.Cxx.getVar('ior')
    baseIorVar = self.Cxx.getVar('baseIor')
    refTypeVar = self.Cxx.getVar('refType')
    returnVar = self.Cxx.getVar('return_value')
    body.addDeclarations([self.Cxx.getDeclaration(returnVar, self.objType)])
    allocCall = self.Cxx.getFunctionCall(self.Cxx.getStructRef(typeVar, 'tp_alloc'), [typeVar, 0])
    selfInit = ASE.Compiler.Cxx.Initializer.Initializer()
    selfInit.setChildren([self.Cxx.castToType(allocCall, self.getSIDLObjectType(interface))])
    selfDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    selfDecl.setIdentifier(selfVar.getIdentifier())
    selfDecl.setType(self.getSIDLObjectType(interface))
    selfDecl.setInitializer(selfInit)
    selfWeakRefList = self.Cxx.getStructRef(selfVar, 'weakRefList')
    setWeakRefList = self.Cxx.getAssignment(selfWeakRefList, self.Cxx.getNull())
    selfIor = self.Cxx.getStructRef(selfVar, 'ior')
    iorObj = self.Cxx.getStructRef(iorVar, 'd_object')
    selfIorEpv = self.Cxx.getStructRef(baseIorVar, 'd_epv')
    trueIor = self.Cxx.getStructRef(baseIorVar, 'd_object')
    iorEpv = self.Cxx.getStructRef(iorVar, 'd_epv')
    castFunc = self.Cxx.getGroup(self.Cxx.getIndirection(self.Cxx.getGroup(self.Cxx.getStructRef(iorEpv, 'castObject'))))
    setSelfIor = self.Cxx.getAssignment(selfIor, self.Cxx.getFunctionCall(self.Cxx.getGroup(castFunc), [iorObj, self.Cxx.getStructRef(typeVar, 'tp_name'), self.Cxx.getNull()]))
    setRefType = self.Cxx.getAssignment(self.Cxx.getStructRef(selfVar, 'refType'), refTypeVar)
    isNewRef = ASE.Compiler.Cxx.Equality.Equality()
    isNewRef.setChildren([refTypeVar, self.Cxx.getVar('NEW_REF')])
    increfCall = self.Cxx.getFunctionCall(self.Cxx.getGroup(self.Cxx.getIndirection(self.Cxx.getGroup(self.Cxx.getStructRef(selfIorEpv, 'addReference')))), [trueIor, self.Cxx.getNull()])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([self.Cxx.getExpStmt(increfCall)])
    checkNewRef = ASE.Compiler.Cxx.If.If()
    checkNewRef.setBranch(isNewRef)
    checkNewRef.setChildren([compound])
    trueBranch = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    baseIorInit = ASE.Compiler.Cxx.Initializer.Initializer()
    baseIorInit.setChildren([self.Cxx.getFunctionCall(castFunc, [iorObj, self.Cxx.getString(self.SIDL.getBaseParent(interface).getFullIdentifier()), self.Cxx.getNull()])])
    trueBranch.addDeclarations([self.Cxx.getDeclaration(baseIorVar, self.IOR.getType(self.SIDL.getBaseParent(interface)), baseIorInit)])
    trueBranch.addChildren([self.Cxx.getExpStmt(setSelfIor), self.Cxx.getExpStmt(setRefType)])
    if interface.isInstanceOf('ASE.Compiler.SIDL.Class'):
      compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
      compound.setChildren([checkNewRef])
      checkRef = ASE.Compiler.Cxx.If.If()
      checkRef.setBranch(self.Cxx.getStructRef(baseIorVar, 'd_data'))
      checkRef.setChildren([compound])
      checkRef.setComments([self.Cxx.getComment('Allow objects without an attached implementation (used in the skeleton constructor)')])
      trueBranch.addChildren([checkRef])
    else:
      trueBranch.addChildren([checkNewRef])
    trueBranch.addChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(returnVar, self.Cxx.castToType(selfVar, self.objType)))])
    falseBranch = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    falseBranch.setChildren([self.Cxx.getExpStmt(self.Cxx.getFunctionCall('PyErr_SetString', ['PyExc_MemoryError', self.Cxx.castToString('Unable to allocate interface object')]))])
    checkSelf = ASE.Compiler.Cxx.If.If()
    checkSelf.setBranch(selfVar)
    checkSelf.setChildren([trueBranch, falseBranch])
    trueBranch = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    trueBranch.setDeclarations([self.Cxx.getDecl(selfDecl)])
    trueBranch.setChildren([self.Cxx.getExpStmt(setWeakRefList), checkSelf])
    falseBranch = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    falseBranch.setChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(returnVar, self.Cxx.getVar('Py_None'))), self.Cxx.getExpStmt(self.Cxx.getFunctionCall('Py_INCREF', [returnVar]))])
    checkIor = ASE.Compiler.Cxx.If.If()
    checkIor.setBranch(iorVar)
    checkIor.setChildren([trueBranch, falseBranch])
    returnObj = ASE.Compiler.Cxx.Return.Return()
    returnObj.setChildren([returnVar])
    body.addChildren([checkIor, returnObj])
    typeParam = ASE.Compiler.Cxx.Parameter.Parameter()
    typeParam.setIdentifier(typeVar.getIdentifier())
    typeParam.setType(self.typeType)
    iorParam = ASE.Compiler.Cxx.Parameter.Parameter()
    iorParam.setIdentifier(iorVar.getIdentifier())
    iorParam.setType(self.IOR.getBaseType(interface))
    refTypeParam = ASE.Compiler.Cxx.Parameter.Parameter()
    refTypeParam.setIdentifier(refTypeVar.getIdentifier())
    refTypeParam.setType(self.refTypeType)
    newName = ASE.Compiler.Cxx.Declarator.Declarator()
    newName.setIdentifier('sidl_new')
    newFunc = ASE.Compiler.Cxx.Function.Function()
    newFunc.setChildren([newName])
    newFunc.setType(self.objType)
    newFunc.setParameters([typeParam, iorParam, refTypeParam])
    newFunc.setBody(body)
    newFunc.setStatic(1)
    return newFunc
  
  def getExtractOpaque(self, type):
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Equality
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.Indirection
    import ASE.Compiler.Cxx.Parameter
    import ASE.Compiler.Cxx.Pointer
    import ASE.Compiler.Cxx.Return
    if type in self.extractIOR:
      return self.extractIOR[type]
    body = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    objVar = self.Cxx.getVar('obj')
    iorVar = self.Cxx.getVar('ptr')
    checkCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    checkCall.setChildren([self.Cxx.getVar('PyCObject_Check')])
    checkCall.setArguments([objVar])
    ptrInd = ASE.Compiler.Cxx.Indirection.Indirection()
    ptrInd.setChildren([iorVar])
    convertCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    convertCall.setChildren([self.Cxx.getVar('PyCObject_AsVoidPtr')])
    convertCall.setArguments([objVar])
    setIor = ASE.Compiler.Cxx.Assignment.Assignment()
    setIor.setChildren([ptrInd, convertCall])
    returnTrue = ASE.Compiler.Cxx.Return.Return()
    returnTrue.setChildren([self.Cxx.getInteger(1)])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([self.Cxx.getExpStmt(setIor), returnTrue])
    checkCObj = ASE.Compiler.Cxx.If.If()
    checkCObj.setBranch(checkCall)
    checkCObj.setChildren([compound])
    hasHandle = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    hasHandle.setChildren([self.Cxx.getVar('PyObject_HasAttrString')])
    hasHandle.setArguments([objVar, self.Cxx.getString('handle')])
    handle = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    handle.setChildren([self.Cxx.getVar('PyObject_GetAttrString')])
    handle.setArguments([objVar, self.Cxx.getString('handle')])
    convertCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    convertCall.setChildren([self.Cxx.getVar('PyLong_AsVoidPtr')])
    convertCall.setArguments([handle])
    setIor = ASE.Compiler.Cxx.Assignment.Assignment()
    setIor.setChildren([ptrInd, convertCall])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([self.Cxx.getExpStmt(setIor), returnTrue])
    checkHandle = ASE.Compiler.Cxx.If.If()
    checkHandle.setBranch(hasHandle)
    checkHandle.setChildren([compound])
    checkHandle.setComments([self.Cxx.getComment('Why in the hell is there no Python interface to CObject?')])
    isNone = ASE.Compiler.Cxx.Equality.Equality()
    isNone.setChildren([objVar, self.Cxx.getVar('Py_None')])
    setIor = ASE.Compiler.Cxx.Assignment.Assignment()
    setIor.setChildren([ptrInd, self.Cxx.getInteger(0)])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([self.Cxx.getExpStmt(setIor), returnTrue])
    checkNone = ASE.Compiler.Cxx.If.If()
    checkNone.setBranch(isNone)
    checkNone.setChildren([compound])
    returnFalse = ASE.Compiler.Cxx.Return.Return()
    returnFalse.setChildren([self.Cxx.getInteger(0)])
    body.addChildren([checkCObj, checkHandle, checkNone, returnFalse])
    objParam = ASE.Compiler.Cxx.Parameter.Parameter()
    objParam.setIdentifier(objVar.getIdentifier())
    objParam.setType(self.objType)
    iorPtrType = ASE.Compiler.Cxx.Pointer.Pointer()
    iorPtrType.setChildren([self.IOR.getType(type)])
    iorParam = ASE.Compiler.Cxx.Parameter.Parameter()
    iorParam.setIdentifier(iorVar.getIdentifier())
    iorParam.setType(iorPtrType)
    extractName = ASE.Compiler.Cxx.Declarator.Declarator()
    extractName.setIdentifier(type.getFullIdentifier().replace('.', '_') + '__extract_ior')
    extractFunc = ASE.Compiler.Cxx.Function.Function()
    extractFunc.setChildren([extractName])
    extractFunc.setType(self.typeMap['int'])
    extractFunc.setStatic(1)
    extractFunc.setParameters([objParam, iorParam])
    extractFunc.setBody(body)
    if not type in self.extractIOR:
      self.extractIOR[type] = extractFunc
    return extractFunc
  
  def getWrapOpaque(self, type):
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.Parameter
    import ASE.Compiler.Cxx.Return
    if type in self.wrapIOR:
      return self.wrapIOR[type]
    body = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    iorVar = self.Cxx.getVar('ptr')
    noneVar = self.Cxx.getVar('Py_None')
    convertCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    convertCall.setChildren([self.Cxx.getVar('PyCObject_FromVoidPtr')])
    convertCall.setArguments([iorVar, self.Cxx.getNull()])
    returnPtr = ASE.Compiler.Cxx.Return.Return()
    returnPtr.setChildren([convertCall])
    trueBranch = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    trueBranch.setChildren([returnPtr])
    increfNone = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    increfNone.setChildren([self.Cxx.getVar('Py_INCREF')])
    increfNone.setArguments([noneVar])
    returnNone = ASE.Compiler.Cxx.Return.Return()
    returnNone.setChildren([noneVar])
    falseBranch = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    falseBranch.setChildren([self.Cxx.getExpStmt(increfNone), returnNone])
    checkPtr = ASE.Compiler.Cxx.If.If()
    checkPtr.setBranch(iorVar)
    checkPtr.setChildren([trueBranch, falseBranch])
    body.addChildren([checkPtr])
    iorParam = ASE.Compiler.Cxx.Parameter.Parameter()
    iorParam.setIdentifier(iorVar.getIdentifier())
    iorParam.setType(self.IOR.getType(type))
    wrapName = ASE.Compiler.Cxx.Declarator.Declarator()
    wrapName.setIdentifier(type.getFullIdentifier().replace('.', '_') + '__wrap_ior')
    wrapFunc = ASE.Compiler.Cxx.Function.Function()
    wrapFunc.setChildren([wrapName])
    wrapFunc.setType(self.objType)
    wrapFunc.setStatic(1)
    wrapFunc.setParameters([iorParam])
    wrapFunc.setBody(body)
    self.wrapIOR[type] = wrapFunc
    return wrapFunc
  
  def getExtractIOR(self, interface, moduleInterface = None):
    '''
    Generate the function which extracts the IOR pointer from a Python object of the given interface
    '''
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Equality
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.Pointer
    if interface in self.extractIOR:
      if moduleInterface is None:
        if isinstance(self.extractIOR[interface], dict):
          return self.extractIOR[interface].values()[0]
        else:
          return self.extractIOR[interface]
      elif moduleInterface in self.extractIOR[interface]:
        return self.extractIOR[interface][moduleInterface]
    body = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    name = interface.getFullIdentifier()
    objVar = self.Cxx.getVar('obj')
    iorVar = self.Cxx.getVar('ior')
    objIorVar = self.Cxx.getVar('objIor')
    baseType = self.IOR.getBaseType(interface)
    body.addDeclarations([self.Cxx.getDeclaration(objIorVar, baseType, self.Cxx.getInitializer(None, self.Cxx.castToType(self.Cxx.getStructRef(objVar, 'ior'), baseType)))])
    noneCmp = ASE.Compiler.Cxx.Equality.Equality()
    noneCmp.setChildren([self.Cxx.castToType(objVar, self.objType), self.Cxx.getVar('Py_None')])
    returnTrue = self.Cxx.getReturn(self.Cxx.getInteger(1))
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(self.Cxx.getIndirection(iorVar), self.Cxx.getNull())), returnTrue])
    checkNone = ASE.Compiler.Cxx.If.If()
    checkNone.setBranch(noneCmp)
    checkNone.setChildren([compound])
    castFunc = self.Cxx.getGroup(self.Cxx.getIndirection(self.Cxx.getGroup(self.Cxx.getStructRef(self.Cxx.getStructRef(objIorVar, 'd_epv'), 'castObject'))))
    castCall = self.Cxx.getFunctionCall(castFunc, [self.Cxx.getStructRef(objIorVar, 'd_object'), self.Cxx.getString(name), self.Cxx.getNull()])
    body.addChildren([checkNone, self.Cxx.getExpStmt(self.Cxx.getAssignment(self.Cxx.getIndirection(iorVar), castCall))])
    body.addChildren([self.getErrorCheck('PyExc_TypeError', 'Argument is not of type ' + name, self.Cxx.getGroup(self.Cxx.getIndirection(iorVar)), self.Cxx.getInteger(0)), returnTrue])
    iorPtrType = ASE.Compiler.Cxx.Pointer.Pointer()
    iorPtrType.setChildren([self.IOR.getType(interface)])
    extractName = ASE.Compiler.Cxx.Declarator.Declarator()
    extractName.setIdentifier(name.replace('.', '_') + '__extract_ior')
    extractFunc = ASE.Compiler.Cxx.Function.Function()
    extractFunc.setChildren([extractName])
    extractFunc.setType(self.typeMap['int'])
    extractFunc.setStatic(1)
    extractFunc.setParameters([self.Cxx.getParameter(objVar.getIdentifier(), self.getSIDLObjectType(moduleInterface)), self.Cxx.getParameter(iorVar.getIdentifier(), iorPtrType)])
    extractFunc.setBody(body)
    if not interface in self.extractIOR:
      self.extractIOR[interface] = {}

    self.extractIOR[interface][moduleInterface] = extractFunc
    return extractFunc
  
  def getWrapIOR(self, interface, isWeak = 0):
    '''
    Generate the function which wraps an IOR pointer in a Python object of the given interface
    '''
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.Initializer
    import ASE.Compiler.Cxx.Parameter
    import ASE.Compiler.Cxx.Return
    if (interface, isWeak) in self.wrapIOR:
      return self.wrapIOR[(interface, isWeak)]
    body = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    iorVar = self.Cxx.getVar('ior')
    moduleVar = self.Cxx.getVar('module')
    typeVar = self.Cxx.getVar('type')
    importCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    importCall.setChildren([self.Cxx.getVar('PyImport_ImportModule')])
    importCall.setArguments([self.Cxx.castToString(interface.getFullIdentifier())])
    moduleInit = ASE.Compiler.Cxx.Initializer.Initializer()
    moduleInit.setChildren([importCall])
    moduleDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    moduleDecl.setIdentifier(moduleVar.getIdentifier())
    moduleDecl.setType(self.objType)
    moduleDecl.setInitializer(moduleInit)
    getAttrCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    getAttrCall.setChildren([self.Cxx.getVar('PyObject_GetAttrString')])
    getAttrCall.setArguments([moduleVar, self.Cxx.castToString(interface.getIdentifier())])
    typeInit = ASE.Compiler.Cxx.Initializer.Initializer()
    typeInit.setChildren([self.Cxx.castToType(getAttrCall, self.typeType)])
    typeDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    typeDecl.setIdentifier(typeVar.getIdentifier())
    typeDecl.setType(self.typeType)
    typeDecl.setInitializer(typeInit)
    body.addDeclarations([self.Cxx.getDecl(moduleDecl), self.Cxx.getDecl(typeDecl)])
    newCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    newCall.setChildren([self.Cxx.getVar('sidl_new')])
    newCall.addArguments([typeVar, self.Cxx.castToType(iorVar, self.IOR.getBaseType(interface))])
    if isWeak:
      newCall.addArguments([self.Cxx.getVar('WEAK_REF')])
    else:
      newCall.addArguments([self.Cxx.getVar('STOLEN_REF')])
    returnNew = ASE.Compiler.Cxx.Return.Return()
    returnNew.setChildren([newCall])
    body.addChildren([self.getErrorCheck('PyExc_ImportError', 'Could not import module ' + interface.getFullIdentifier(), moduleVar, self.Cxx.getInteger(0)), self.getErrorCheck('PyExc_TypeError', 'Could not find type ' + interface.getFullIdentifier(), typeVar, self.Cxx.getInteger(0)), returnNew])
    iorParam = ASE.Compiler.Cxx.Parameter.Parameter()
    iorParam.setIdentifier(iorVar.getIdentifier())
    iorParam.setType(self.IOR.getType(interface))
    wrapName = ASE.Compiler.Cxx.Declarator.Declarator()
    if isWeak:
      wrapName.setIdentifier(interface.getFullIdentifier().replace('.', '_') + '__wrap_ior_weak')
    else:
      wrapName.setIdentifier(interface.getFullIdentifier().replace('.', '_') + '__wrap_ior')
    wrapFunc = ASE.Compiler.Cxx.Function.Function()
    wrapFunc.setChildren([wrapName])
    wrapFunc.setType(self.objType)
    wrapFunc.setStatic(1)
    wrapFunc.setParameters([iorParam])
    wrapFunc.setBody(body)
    self.wrapIOR[(interface, isWeak)] = wrapFunc
    return wrapFunc
  
  def getPythonArrayInfo(self, body):
    import ASE.Compiler.SIDL.Array
    import ASE.Compiler.Cxx.Array
    import ASE.Compiler.Cxx.ArrayReference
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.For
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.Increment
    import ASE.Compiler.Cxx.Initializer
    import ASE.Compiler.Cxx.LessThan
    import ASE.Compiler.Cxx.Multiplication
    import ASE.Compiler.Cxx.Return
    import ASE.Compiler.Cxx.Sizeof
    import ASE.Compiler.Cxx.Struct
    import ASE.Compiler.Cxx.StructReference
    import ASE.Compiler.Cxx.Subtraction
    pyArrayVar = self.Cxx.getVar('pyArray')
    dimenVar = self.Cxx.getVar('dimen')
    indLowerVar = self.Cxx.getVar('indLower')
    indUpperVar = self.Cxx.getVar('indUpper')
    indStrideVar = self.Cxx.getVar('indStride')
    lowerVar = self.Cxx.getVar('lower')
    upperVar = self.Cxx.getVar('upper')
    strideVar = self.Cxx.getVar('stride')
    dVar = self.Cxx.getVar('d')
    freeVar = self.Cxx.getVar('free')
    intDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    intDecl.setType(self.IOR.typeMap['int'])
    sizeofInt = ASE.Compiler.Cxx.Sizeof.Sizeof()
    sizeofInt.setTypeName(intDecl)
    intArray = ASE.Compiler.SIDL.Array.Array()
    intArray.setType(self.SIDL.typeMap['int'])
    intStruct = ASE.Compiler.Cxx.Struct.Struct(self.IOR.getType(intArray).getChildren()[0])
    pyArrayDimen = ASE.Compiler.Cxx.StructReference.StructReference()
    pyArrayDimen.setChildren([pyArrayVar])
    pyArrayDimen.setMember('nd')
    pyArrayDimen.setPointer(1)
    pyArrayUpper = ASE.Compiler.Cxx.StructReference.StructReference()
    pyArrayUpper.setChildren([pyArrayVar])
    pyArrayUpper.setMember('dimensions')
    pyArrayUpper.setPointer(1)
    pyArrayStride = ASE.Compiler.Cxx.StructReference.StructReference()
    pyArrayStride.setChildren([pyArrayVar])
    pyArrayStride.setMember('strides')
    pyArrayStride.setPointer(1)
    pyArrayDesc = ASE.Compiler.Cxx.StructReference.StructReference()
    pyArrayDesc.setChildren([pyArrayVar])
    pyArrayDesc.setMember('descr')
    pyArrayDesc.setPointer(1)
    pyArrayElsize = ASE.Compiler.Cxx.StructReference.StructReference()
    pyArrayElsize.setChildren([pyArrayDesc])
    pyArrayElsize.setMember('elsize')
    pyArrayElsize.setPointer(1)
    dPyArrayUpper = ASE.Compiler.Cxx.ArrayReference.ArrayReference()
    dPyArrayUpper.setChildren([pyArrayUpper])
    dPyArrayUpper.setIndex(dVar)
    dPyArrayStride = ASE.Compiler.Cxx.ArrayReference.ArrayReference()
    dPyArrayStride.setChildren([pyArrayStride])
    dPyArrayStride.setIndex(dVar)
    lowerFirst = ASE.Compiler.Cxx.StructReference.StructReference()
    lowerFirst.setChildren([lowerVar])
    lowerFirst.setMember('d_firstElement')
    upperFirst = ASE.Compiler.Cxx.StructReference.StructReference()
    upperFirst.setChildren([upperVar])
    upperFirst.setMember('d_firstElement')
    strideFirst = ASE.Compiler.Cxx.StructReference.StructReference()
    strideFirst.setChildren([strideVar])
    strideFirst.setMember('d_firstElement')
    dLowerFirst = ASE.Compiler.Cxx.ArrayReference.ArrayReference()
    dLowerFirst.setChildren([lowerFirst])
    dLowerFirst.setIndex(dVar)
    dUpperFirst = ASE.Compiler.Cxx.ArrayReference.ArrayReference()
    dUpperFirst.setChildren([upperFirst])
    dUpperFirst.setIndex(dVar)
    dStrideFirst = ASE.Compiler.Cxx.ArrayReference.ArrayReference()
    dStrideFirst.setChildren([strideFirst])
    dStrideFirst.setIndex(dVar)
    dimenInit = ASE.Compiler.Cxx.Initializer.Initializer()
    dimenInit.setChildren([pyArrayDimen])
    indLowerInit = ASE.Compiler.Cxx.Initializer.Initializer()
    indLowerInit.setChildren([self.Cxx.getInteger(0)])
    indLowerInit.setList(1)
    indLower = ASE.Compiler.Cxx.Array.Array()
    indLower.setChildren([indLowerVar])
    indLower.setType(self.IOR.typeMap['int'])
    indLower.setSize(self.Cxx.getInteger(1))
    indLower.setInitializer(indLowerInit)
    dimenMinus1 = ASE.Compiler.Cxx.Subtraction.Subtraction()
    dimenMinus1.setChildren([dimenVar, self.Cxx.getInteger(1)])
    indUpperInit = ASE.Compiler.Cxx.Initializer.Initializer()
    indUpperInit.setChildren([dimenMinus1])
    indUpperInit.setList(1)
    indUpper = ASE.Compiler.Cxx.Array.Array()
    indUpper.setChildren([indUpperVar])
    indUpper.setType(self.IOR.typeMap['int'])
    indUpper.setSize(self.Cxx.getInteger(1))
    indUpper.setInitializer(indUpperInit)
    indStrideInit = ASE.Compiler.Cxx.Initializer.Initializer()
    indStrideInit.setChildren([self.Cxx.getInteger(1)])
    indStrideInit.setList(1)
    indStride = ASE.Compiler.Cxx.Array.Array()
    indStride.setChildren([indStrideVar])
    indStride.setType(self.IOR.typeMap['int'])
    indStride.setSize(self.Cxx.getInteger(1))
    indStride.setInitializer(indStrideInit)
    safeDimen = ASE.Compiler.Cxx.If.If()
    safeDimen.setBranch(dimenVar)
    safeDimen.setChildren([dimenVar, self.Cxx.getInteger(1)])
    safeDimen.setTernary(1)
    indexSize = ASE.Compiler.Cxx.Multiplication.Multiplication()
    indexSize.setChildren([sizeofInt, self.Cxx.getGroup(safeDimen)])
    indexAlloc = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    indexAlloc.setChildren([self.Cxx.getVar('malloc')])
    indexAlloc.setArguments([indexSize])
    indexInit = ASE.Compiler.Cxx.Initializer.Initializer()
    indexInit.setChildren([indexAlloc, indLowerVar, indUpperVar, indStrideVar, self.Cxx.getInteger(0), self.Cxx.getInteger(1)])
    body.addDeclarations([self.Cxx.getDeclaration(dimenVar, self.IOR.typeMap['int'], dimenInit)])
    body.addDeclarations(map(self.Cxx.getDecl, [indLower, indUpper, indStride]))
    body.addDeclarations([self.Cxx.getDeclaration(lowerVar, intStruct, indexInit), self.Cxx.getDeclaration(upperVar, intStruct, indexInit), self.Cxx.getDeclaration(strideVar, intStruct, indexInit), self.Cxx.getDeclaration(dVar, self.IOR.typeMap['int'])])
    setLower = ASE.Compiler.Cxx.Assignment.Assignment()
    setLower.setChildren([dLowerFirst, self.Cxx.getInteger(0)])
    newUpper = ASE.Compiler.Cxx.Subtraction.Subtraction()
    newUpper.setChildren([dPyArrayUpper, self.Cxx.getInteger(1)])
    setUpper = ASE.Compiler.Cxx.Assignment.Assignment()
    setUpper.setChildren([dUpperFirst, newUpper])
    newStride = ASE.Compiler.Cxx.Division.Division()
    newStride.setChildren([dPyArrayStride, pyArrayElsize])
    setStride = ASE.Compiler.Cxx.Assignment.Assignment()
    setStride.setChildren([dStrideFirst, newStride])
    freeLower = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    freeLower.setChildren([freeVar])
    freeLower.setArguments([lowerFirst])
    freeUpper = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    freeUpper.setChildren([freeVar])
    freeUpper.setArguments([upperFirst])
    freeStride = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    freeStride.setChildren([freeVar])
    freeStride.setArguments([strideFirst])
    returnFail = ASE.Compiler.Cxx.Return.Return()
    returnFail.setChildren([self.Cxx.getInteger(0)])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.addChildren(map(self.Cxx.getExpStmt, [freeLower, freeUpper, freeStride]))
    compound.addChildren([returnFail])
    extraSpace = ASE.Compiler.Cxx.Modulo.Modulo()
    extraSpace.setChildren([dPyArrayStride, pyArrayElsize])
    checkSize = ASE.Compiler.Cxx.If.If()
    checkSize.setBranch(extraSpace)
    checkSize.setChildren([compound])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren(map(self.Cxx.getExpStmt, [setLower, setUpper, setStride]) + [checkSize])
    dInit = ASE.Compiler.Cxx.Assignment.Assignment()
    dInit.setChildren([dVar, self.Cxx.getInteger(0)])
    dCmp = ASE.Compiler.Cxx.LessThan.LessThan()
    dCmp.setChildren([dVar, dimenVar])
    dInc = ASE.Compiler.Cxx.Increment.Increment()
    dInc.setChildren([dVar])
    dInc.setPostfix(1)
    indexLoop = ASE.Compiler.Cxx.For.For()
    indexLoop.setInitialization(dInit)
    indexLoop.setBranch(dCmp)
    indexLoop.setIncrement(dInc)
    indexLoop.setChildren([compound])
    body.addChildren([indexLoop])
    return

  def canBorrowArray(self, type):
    if type.getBaseType() and not (type.getIdentifier() in ['dcomplex', 'fcomplex', 'opaque', 'string']):
      return 1
    return 0

  def getExtractArray(self, type, borrow, moduleInterface = None):
    import ASE.Compiler.SIDL.Array
    import ASE.Compiler.Cxx.AdditionAssignment
    import ASE.Compiler.Cxx.ArrayReference
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.Break
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.For
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.Increment
    import ASE.Compiler.Cxx.Indirection
    import ASE.Compiler.Cxx.Initializer
    import ASE.Compiler.Cxx.LessThan
    import ASE.Compiler.Cxx.Multiplication
    import ASE.Compiler.Cxx.Negation
    import ASE.Compiler.Cxx.Parameter
    import ASE.Compiler.Cxx.Pointer
    import ASE.Compiler.Cxx.Return
    import ASE.Compiler.Cxx.StructReference
    import ASE.Compiler.Cxx.SubtractionAssignment

    if borrow and not self.canBorrowArray(type):
      raise RuntimeError('Cannot wrap a borrowed array from type '+type.getFullIdentifier())
    if type in self.extractArray:
      if moduleInterface in self.extractArray[type][borrow]:
        return self.extractArray[type][borrow][moduleInterface]
      elif moduleInterface is None and len(self.extractArray[type][borrow].keys()):
        return self.extractArray[type][borrow].values()[0]
    else:
      self.extractArray[type] = [{}, {}]
    body = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    objVar = self.Cxx.getVar('obj')
    arrayVar = self.Cxx.getVar('array')
    pyArrayVar = self.Cxx.getVar('pyArray')
    returnVar = self.Cxx.getVar('_return_value')
    arrayType = ASE.Compiler.SIDL.Array.Array()
    arrayType.setType(type)
    intArray = ASE.Compiler.SIDL.Array.Array()
    intArray.setType(self.SIDL.typeMap['int'])
    pyArrayInit = self.Cxx.getInitializer(None, self.Cxx.castToType(self.Cxx.getFunctionCall('PyArray_FromObject', [objVar, self.getNumericTypeCode(type), 0, 0]), self.arrayType))
    pyArrayDecl = self.Cxx.getDeclaration(pyArrayVar, self.arrayType, pyArrayInit, 'This will create a new Numeric array from a Python sequence')
    returnDecl = self.Cxx.getDeclaration(returnVar, self.typeMap['int'])
    body.addDeclarations([pyArrayDecl, returnDecl])
    dimenVar = self.Cxx.getVar('dimen')
    lowerVar = self.Cxx.getVar('lower')
    upperVar = self.Cxx.getVar('upper')
    strideVar = self.Cxx.getVar('stride')
    freeVar = self.Cxx.getVar('free')
    intDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    intDecl.setType(self.typeMap['int'])
    intPtr = ASE.Compiler.Cxx.Pointer.Pointer()
    intPtr.setChildren([self.typeMap['int']])
    charPtr = ASE.Compiler.Cxx.Pointer.Pointer()
    charPtr.setChildren([self.typeMap['char']])
    arrayInd = self.Cxx.getIndirection(arrayVar)
    arrayInit = self.Cxx.getExpStmt(self.Cxx.getAssignment(arrayInd, self.Cxx.getNull()))
    lowerFirst = ASE.Compiler.Cxx.StructReference.StructReference()
    lowerFirst.setChildren([lowerVar])
    lowerFirst.setMember('d_firstElement')
    upperFirst = ASE.Compiler.Cxx.StructReference.StructReference()
    upperFirst.setChildren([upperVar])
    upperFirst.setMember('d_firstElement')
    strideFirst = ASE.Compiler.Cxx.StructReference.StructReference()
    strideFirst.setChildren([strideVar])
    strideFirst.setMember('d_firstElement')
    mainBody = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    self.getPythonArrayInfo(mainBody)
    externalsVar = self.Cxx.getVar('_externals')
    extFuncVar = self.Cxx.getVar('_extFunc')
    sizeVar = self.Cxx.getVar('size')
    indicesVar = self.Cxx.getVar('indices')
    dataVar = self.Cxx.getVar('data')
    iVar = self.Cxx.getVar('i')
    dVar = self.Cxx.getVar('d')
    freejVar = self.Cxx.getVar('free')

    if type.getBaseType():
      externalsCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      externalsCall.setChildren([self.Cxx.getVar(type.getFullIdentifier().replace('.', '_')+ '__externals')])
    else:
      externalsFuncType = ASE.Compiler.Cxx.Function.Function()
      externalsFuncType.setChildren([self.Cxx.getDeclGroup(ASE.Compiler.Cxx.Pointer.Pointer())])
      externalsFuncType.setType(self.IOR.getExternalType(type))
      sidlExtCall       = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      sidlExtCall.setChildren([self.Cxx.getVar('ASE_Loader__externals')])
      staticEPV         = ASE.Compiler.Cxx.StructReference.StructReference()
      staticEPV.setChildren([sidlExtCall])
      staticEPV.setMember('getStaticEPV')
      staticEPV.setPointer(1)
      staticEPVCall     = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      staticEPVCall.setChildren([staticEPV])
      staticEPVCall.setArguments([self.Cxx.getNull()])
      classMethods      = ASE.Compiler.Cxx.StructReference.StructReference()
      classMethods.setChildren([staticEPVCall])
      classMethods.setMember('getClassMethodTable')
      classMethods.setPointer(1)
      classMethodsCall  = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      classMethodsCall.setChildren([classMethods])
      classMethodsCall.setArguments([self.Cxx.getString(type.getFullIdentifier()), self.Cxx.getNull()])
      externalsFunc     = ASE.Compiler.Cxx.Cast.Cast()
      externalsFunc.setTypeName(externalsFuncType)
      externalsFunc.setChildren([classMethodsCall])
      externalsInit = ASE.Compiler.Cxx.Initializer.Initializer()
      externalsInit.setChildren([externalsFunc])
      externalsFuncPtr = ASE.Compiler.Cxx.Declarator.Declarator()
      externalsFuncPtr.setIdentifier(externalsVar.getIdentifier())
      externalsFuncPtr.setType(ASE.Compiler.Cxx.Pointer.Pointer())
      externalsFunc = ASE.Compiler.Cxx.Function.Function()
      externalsFunc.setChildren([self.Cxx.getDeclGroup(externalsFuncPtr)])
      externalsFunc.setType(self.IOR.getExternalType(type))
      externalsFunc.setInitializer(externalsInit)
      mainBody.addDeclarations([self.Cxx.getDecl(externalsFunc)])
      externalsInd  = ASE.Compiler.Cxx.Indirection.Indirection()
      externalsInd.setChildren([externalsVar])
      externalsCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      externalsCall.setChildren([self.Cxx.getGroup(externalsInd)])
    extFuncInit = ASE.Compiler.Cxx.Initializer.Initializer()
    extFuncInit.setChildren([externalsCall])
    extFuncDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    extFuncDecl.setIdentifier(extFuncVar.getIdentifier())
    extFuncDecl.setType(self.IOR.getExternalType(type))
    extFuncDecl.setInitializer(extFuncInit)
    arraySize = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    arraySize.setChildren([self.Cxx.getVar('PyArray_Size')])
    arraySize.setArguments([self.Cxx.castToType(pyArrayVar, self.objType)])
    sizeInit = ASE.Compiler.Cxx.Initializer.Initializer()
    sizeInit.setChildren([arraySize])
    sizeDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    sizeDecl.setIdentifier(sizeVar.getIdentifier())
    sizeDecl.setType(self.typeMap['int'])
    sizeDecl.setInitializer(sizeInit)
    sizeofInt = ASE.Compiler.Cxx.Sizeof.Sizeof()
    sizeofInt.setTypeName(intDecl)
    safeDimen = ASE.Compiler.Cxx.If.If()
    safeDimen.setBranch(dimenVar)
    safeDimen.setChildren([dimenVar, self.Cxx.getInteger(1)])
    safeDimen.setTernary(1)
    indicesSize = ASE.Compiler.Cxx.Multiplication.Multiplication()
    indicesSize.setChildren([sizeofInt, self.Cxx.getGroup(safeDimen)])
    indicesMalloc = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    indicesMalloc.setChildren([self.Cxx.getVar('malloc')])
    indicesMalloc.setArguments([indicesSize])
    pyArrayData = ASE.Compiler.Cxx.StructReference.StructReference()
    pyArrayData.setChildren([pyArrayVar])
    pyArrayData.setMember('data')
    pyArrayData.setPointer(1)
    dataInit = ASE.Compiler.Cxx.Initializer.Initializer()
    dataInit.setChildren([pyArrayData])
    dataDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    dataDecl.setIdentifier(dataVar.getIdentifier())
    dataDecl.setType(charPtr)
    dataDecl.setInitializer(dataInit)
    iDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    iDecl.setIdentifier(iVar.getIdentifier())
    iDecl.setType(self.typeMap['int'])
    if not borrow:
      createFuncPtr = ASE.Compiler.Cxx.StructReference.StructReference()
      createFuncPtr.setChildren([extFuncVar])
      createFuncPtr.setMember('createArray')
      createFuncPtr.setPointer(1)
      createFunc = ASE.Compiler.Cxx.Indirection.Indirection()
      createFunc.setChildren([createFuncPtr])
      createArray = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      createArray.setChildren([self.Cxx.getGroup(createFunc)])
      createArray.setArguments([dimenVar, self.Cxx.getAddress(lowerVar), self.Cxx.getAddress(upperVar)])
      createArray.addArguments([self.Cxx.getNull()])
      setArray = ASE.Compiler.Cxx.Assignment.Assignment()
      setArray.setChildren([arrayInd, createArray])
    else:
      borrowFuncPtr = ASE.Compiler.Cxx.StructReference.StructReference()
      borrowFuncPtr.setChildren([extFuncVar])
      borrowFuncPtr.setMember('borrowArray')
      borrowFuncPtr.setPointer(1)
      borrowFunc = ASE.Compiler.Cxx.Indirection.Indirection()
      borrowFunc.setChildren([borrowFuncPtr])
      borrowArray = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      borrowArray.setChildren([self.Cxx.getGroup(borrowFunc)])
      borrowArray.setArguments([dimenVar, self.Cxx.getAddress(lowerVar), self.Cxx.getAddress(upperVar), self.Cxx.getAddress(strideVar), dataVar])
      borrowArray.addArguments([self.Cxx.getNull()])
      setArray = ASE.Compiler.Cxx.Assignment.Assignment()
      setArray.setChildren([arrayInd, borrowArray])
    freeLower = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    freeLower.setChildren([freeVar])
    freeLower.setArguments([lowerFirst])
    freeUpper = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    freeUpper.setChildren([freeVar])
    freeUpper.setArguments([upperFirst])
    freeStride = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    freeStride.setChildren([freeVar])
    freeStride.setArguments([strideFirst])
    indData = ASE.Compiler.Cxx.StructReference.StructReference()
    indData.setChildren([indicesVar])
    indData.setMember('d_firstElement')
    mainBody.addDeclarations(map(self.Cxx.getDecl, [extFuncDecl, sizeDecl, dataDecl, iDecl]))
    mainBody.addChildren(map(self.Cxx.getExpStmt, [setArray, freeLower, freeUpper, freeStride]))
    if not borrow:
      indLowerVar = self.Cxx.getVar('indLower')
      indUpperVar = self.Cxx.getVar('indUpper')
      indStrideVar = self.Cxx.getVar('indStride')
      indicesInit = ASE.Compiler.Cxx.Initializer.Initializer()
      indicesInit.setChildren([indicesMalloc, indLowerVar, indUpperVar, indStrideVar, self.Cxx.getInteger(0), self.Cxx.getInteger(1)])
      indicesDecl = ASE.Compiler.Cxx.Declarator.Declarator()
      indicesDecl.setIdentifier(indicesVar.getIdentifier())
      indicesDecl.setType(self.IOR.getType(intArray).getChildren()[0])
      indicesDecl.setInitializer(indicesInit)
      mainBody.addDeclarations(map(self.Cxx.getDecl, [indicesDecl]))
      clearIndices = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      clearIndices.setChildren([self.Cxx.getVar('memset')])
      clearIndices.setArguments([indData, self.Cxx.getInteger(0), indicesSize])
      mainBody.addChildren(map(self.Cxx.getExpStmt, [clearIndices]))
      elemVar = self.Cxx.getVar('elem')
      elemDecl = ASE.Compiler.Cxx.Declarator.Declarator()
      elemDecl.setIdentifier(elemVar.getIdentifier())
      elemDecl.setType(self.IOR.getType(type))
      destroyFuncPtr = ASE.Compiler.Cxx.StructReference.StructReference()
      destroyFuncPtr.setChildren([extFuncVar])
      destroyFuncPtr.setMember('destroyArray')
      destroyFuncPtr.setPointer(1)
      destroyFunc = ASE.Compiler.Cxx.Indirection.Indirection()
      destroyFunc.setChildren([destroyFuncPtr])
      destroyArray = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      destroyArray.setChildren([self.Cxx.getGroup(destroyFunc)])
      destroyArray.setArguments([arrayInd, self.Cxx.getNull()])
      gotoError = ASE.Compiler.Cxx.Goto.Goto()
      gotoError.setIdentifier('error')
      compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
      compound.setDeclarations([self.Cxx.getDecl(elemDecl)])
      if type.getBaseType():
        typeName = type.getIdentifier()
        if typeName == 'string':
          dataInd = ASE.Compiler.Cxx.Indirection.Indirection()
          dataInd.setChildren([self.Cxx.castToType(dataVar, self.objType, 1)])
          cmpd = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
          cmpd.setChildren([self.Cxx.getExpStmt(destroyArray), arrayInit, gotoError])
          convertCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
          convertCall.setChildren([self.Cxx.getVar('PyString_AsString')])
          convertCall.setArguments([dataInd])
          setElem = ASE.Compiler.Cxx.Assignment.Assignment()
          setElem.setChildren([elemVar, convertCall])
          negConvert = ASE.Compiler.Cxx.Negation.Negation()
          negConvert.setChildren([self.Cxx.getGroup(setElem)])
          convertElem = ASE.Compiler.Cxx.If.If()
          convertElem.setBranch(negConvert)
          convertElem.setChildren([cmpd])
          compound.addChildren([convertElem])
          self.copyValue_string(elemVar, compound)
        elif typeName == 'opaque':
          dataInd = ASE.Compiler.Cxx.Indirection.Indirection()
          dataInd.setChildren([self.Cxx.castToType(dataVar, self.objType, 1)])
          cmpd = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
          cmpd.setChildren([self.Cxx.getExpStmt(destroyArray), arrayInit, gotoError])
          convertCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
          convertCall.setChildren([self.Cxx.getVar('PyCObject_AsVoidPtr')])
          convertCall.setArguments([dataInd])
          setElem = ASE.Compiler.Cxx.Assignment.Assignment()
          setElem.setChildren([elemVar, convertCall])
          negConvert = ASE.Compiler.Cxx.Negation.Negation()
          negConvert.setChildren([self.Cxx.getGroup(setElem)])
          convertElem = ASE.Compiler.Cxx.If.If()
          convertElem.setBranch(negConvert)
          convertElem.setChildren([cmpd])
          compound.addChildren([convertElem])
        elif typeName in ['dcomplex', 'fcomplex']:
          dataInd = ASE.Compiler.Cxx.Indirection.Indirection()
          dataInd.setChildren([self.Cxx.castToType(dataVar, self.IOR.getType(type), 1)])
          self.setValue_dcomplex(elemVar, dataInd, compound, setIOR = 1, pythonImagName = 'imaginary')
        else:
          dataInd = ASE.Compiler.Cxx.Indirection.Indirection()
          dataInd.setChildren([self.Cxx.castToType(dataVar, self.getType(type), 1)])
          convertElem = ASE.Compiler.Cxx.Assignment.Assignment()
          convertElem.setChildren([elemVar, dataInd])
          compound.addChildren([self.Cxx.getExpStmt(convertElem)])
      elif type.isInstanceOf('ASE.Compiler.SIDL.Interface'):
        dataInd = ASE.Compiler.Cxx.Indirection.Indirection()
        dataInd.setChildren([self.Cxx.castToType(dataVar, self.objType, 1)])
        cmpd = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
        cmpd.setChildren([self.Cxx.getExpStmt(destroyArray), arrayInit, gotoError])
        convertCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
        convertCall.setChildren([self.Cxx.getVar(type.getFullIdentifier().replace('.', '_') + '__extract_ior')])
        convertCall.setArguments([self.Cxx.castToType(dataInd, self.getSIDLObjectType(moduleInterface)), self.Cxx.getAddress(elemVar)])
        negConvert = ASE.Compiler.Cxx.Negation.Negation()
        negConvert.setChildren([convertCall])
        convertElem = ASE.Compiler.Cxx.If.If()
        convertElem.setBranch(negConvert)
        convertElem.setChildren([cmpd])
        compound.addChildren([convertElem])
      setFuncPtr = ASE.Compiler.Cxx.StructReference.StructReference()
      setFuncPtr.setChildren([extFuncVar])
      setFuncPtr.setMember('setElement')
      setFuncPtr.setPointer(1)
      setFunc = ASE.Compiler.Cxx.Indirection.Indirection()
      setFunc.setChildren([setFuncPtr])
      setElement = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      setElement.setChildren([self.Cxx.getGroup(setFunc)])
      setElement.setArguments([arrayInd, self.Cxx.getAddress(indicesVar), elemVar])
      setElement.addArguments([self.Cxx.getNull()])
      strides = ASE.Compiler.Cxx.StructReference.StructReference()
      strides.setChildren([pyArrayVar])
      strides.setMember('strides')
      strides.setPointer(1)
      dimStride = ASE.Compiler.Cxx.ArrayReference.ArrayReference()
      dimStride.setChildren([strides])
      dimStride.setIndex(dVar)
      incData = ASE.Compiler.Cxx.AdditionAssignment.AdditionAssignment()
      incData.setChildren([dataVar, dimStride])
      lengths = ASE.Compiler.Cxx.StructReference.StructReference()
      lengths.setChildren([pyArrayVar])
      lengths.setMember('dimensions')
      lengths.setPointer(1)
      dimLength = ASE.Compiler.Cxx.ArrayReference.ArrayReference()
      dimLength.setChildren([lengths])
      dimLength.setIndex(dVar)
      dimIndex = ASE.Compiler.Cxx.ArrayReference.ArrayReference()
      dimIndex.setChildren([indData])
      dimIndex.setIndex(dVar)
      incDimIndex = ASE.Compiler.Cxx.Increment.Increment()
      incDimIndex.setChildren([self.Cxx.getGroup(dimIndex)])
      compLength = ASE.Compiler.Cxx.LessThan.LessThan()
      compLength.setChildren([incDimIndex, dimLength])
      trueBranch = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
      trueBranch.setChildren([ASE.Compiler.Cxx.Break.Break()])
      resetIndex = ASE.Compiler.Cxx.Assignment.Assignment()
      resetIndex.setChildren([dimIndex, self.Cxx.getInteger(0)])
      dimSize = ASE.Compiler.Cxx.Multiplication.Multiplication()
      dimSize.setChildren([dimLength, dimStride])
      decData = ASE.Compiler.Cxx.SubtractionAssignment.SubtractionAssignment()
      decData.setChildren([dataVar, dimSize])
      falseBranch = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
      falseBranch.setChildren([self.Cxx.getExpStmt(resetIndex), self.Cxx.getExpStmt(decData)])
      checkWrap = ASE.Compiler.Cxx.If.If()
      checkWrap.setBranch(compLength)
      checkWrap.setChildren([trueBranch, falseBranch])
      cmpd = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
      cmpd.setChildren([self.Cxx.getExpStmt(incData), checkWrap])
      initDim = ASE.Compiler.Cxx.Assignment.Assignment()
      initDim.setChildren([dVar, self.Cxx.getInteger(0)])
      checkDim = ASE.Compiler.Cxx.LessThan.LessThan()
      checkDim.setChildren([dVar, dimenVar])
      incDim = ASE.Compiler.Cxx.Increment.Increment()
      incDim.setChildren([dVar])
      incDim.setPostfix(1)
      updateLocation = ASE.Compiler.Cxx.For.For()
      updateLocation.setInitialization(initDim)
      updateLocation.setBranch(checkDim)
      updateLocation.setIncrement(incDim)
      updateLocation.setChildren([cmpd])
      compound.addChildren([self.Cxx.getExpStmt(setElement), updateLocation])
      initIndex = ASE.Compiler.Cxx.Assignment.Assignment()
      initIndex.setChildren([iVar, self.Cxx.getInteger(0)])
      checkIndex = ASE.Compiler.Cxx.LessThan.LessThan()
      checkIndex.setChildren([iVar, sizeVar])
      incIndex = ASE.Compiler.Cxx.Increment.Increment()
      incIndex.setChildren([iVar])
      incIndex.setPostfix(1)
      elementLoop = ASE.Compiler.Cxx.For.For()
      elementLoop.setInitialization(initIndex)
      elementLoop.setBranch(checkIndex)
      elementLoop.setIncrement(incIndex)
      elementLoop.setChildren([compound])
      mainBody.addChildren([elementLoop])
    setReturn = ASE.Compiler.Cxx.Assignment.Assignment()
    setReturn.setChildren([returnVar, self.Cxx.getInteger(1)])
    mainBody.addChildren([self.Cxx.getExpStmt(setReturn)])
    if not borrow:
      freeIndices = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      freeIndices.setChildren([freeVar])
      freeIndices.setArguments([indData])
      freeIndices = self.Cxx.getExpStmt(freeIndices)
      freeIndices.setIdentifier('error')
      mainBody.addChildren([freeIndices])
      decrefPyArray = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      decrefPyArray.setChildren([self.Cxx.getVar('Py_DECREF')])
      decrefPyArray.setArguments([pyArrayVar])
      mainBody.addChildren([self.Cxx.getExpStmt(decrefPyArray)])
    checkPyArray = ASE.Compiler.Cxx.If.If()
    checkPyArray.setBranch(pyArrayVar)
    checkPyArray.setChildren([mainBody])
    returnStmt = ASE.Compiler.Cxx.Return.Return()
    returnStmt.setChildren([returnVar])
    body.addChildren([arrayInit, checkPyArray, returnStmt])
    objParam = ASE.Compiler.Cxx.Parameter.Parameter()
    objParam.setIdentifier(objVar.getIdentifier())
    objParam.setType(self.objType)
    arrayPtrType = ASE.Compiler.Cxx.Pointer.Pointer()
    arrayPtrType.setChildren([self.IOR.getType(arrayType)])
    arrayParam = ASE.Compiler.Cxx.Parameter.Parameter()
    arrayParam.setIdentifier(arrayVar.getIdentifier())
    arrayParam.setType(arrayPtrType)
    extractName = ASE.Compiler.Cxx.Declarator.Declarator()
    extractName.setIdentifier(type.getFullIdentifier().replace('.', '_') + '__extract_array')
    extractFunc = ASE.Compiler.Cxx.Function.Function()
    extractFunc.setChildren([extractName])
    extractFunc.setType(self.typeMap['int'])
    extractFunc.setStatic(1)
    extractFunc.setParameters([objParam, arrayParam])
    extractFunc.setBody(body)
    self.extractArray[type][borrow][moduleInterface] = extractFunc
    return extractFunc

  def getWrapArray(self, type, borrow):
    import ASE.Compiler.SIDL.Array
    import ASE.Compiler.Cxx.Array
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Goto
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.MultiplicationAssignment
    import ASE.Compiler.Cxx.While

    if borrow and not self.canBorrowArray(type):
      raise RuntimeError('Cannot wrap a borrowed array from type '+type.getFullIdentifier())
    if (type, borrow) in self.wrapArray:
      return self.wrapArray[(type, borrow)]
    arrayType = ASE.Compiler.SIDL.Array.Array()
    arrayType.setType(type)
    intArray = ASE.Compiler.SIDL.Array.Array()
    intArray.setType(self.SIDL.typeMap['int'])
    body = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    name = type.getFullIdentifier()
    null = self.Cxx.getNull()
    arrayVar = self.Cxx.getVar('array')
    returnVar = self.Cxx.getVar('_return_value')

    body.addDeclarations([self.Cxx.getDeclaration(returnVar, self.arrayType)])
    mainBody      = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()

    externalsVar  = self.Cxx.getVar('_externals')
    extFuncVar    = self.Cxx.getVar('_extFunc')
    dimenVar      = self.Cxx.getVar('dimen')
    lowerVar      = self.Cxx.getVar('lower')
    upperVar      = self.Cxx.getVar('upper')
    numElemVar    = self.Cxx.getVar('numElem')
    iVar          = self.Cxx.getVar('i')

    if type.getBaseType():
      externalsCall = self.Cxx.getFunctionCall(self.Cxx.getVar(name.replace('.', '_')+ '__externals'))
    else:
      staticEPVCall = self.Cxx.getFunctionCall(self.Cxx.getStructRef(self.Cxx.getFunctionCall('ASE_Loader__externals'), 'getStaticEPV'), [null])
      classMethodsCall = self.Cxx.getFunctionCall(self.Cxx.getStructRef(staticEPVCall, 'getClassMethodTable'), [self.Cxx.getString(type.getFullIdentifier()), null])
      externalsFuncType = ASE.Compiler.Cxx.Function.Function()
      externalsFuncType.setChildren([self.Cxx.getDeclGroup(ASE.Compiler.Cxx.Pointer.Pointer())])
      externalsFuncType.setType(self.IOR.getExternalType(type))
      externalsFunc = ASE.Compiler.Cxx.Cast.Cast()
      externalsFunc.setTypeName(externalsFuncType)
      externalsFunc.setChildren([classMethodsCall])
      externalsInit = self.Cxx.getInitializer(None, externalsFunc)
      externalsFuncPtr = ASE.Compiler.Cxx.Declarator.Declarator()
      externalsFuncPtr.setIdentifier(externalsVar.getIdentifier())
      externalsFuncPtr.setType(ASE.Compiler.Cxx.Pointer.Pointer())
      externalsFunc = ASE.Compiler.Cxx.Function.Function()
      externalsFunc.setChildren([self.Cxx.getDeclGroup(externalsFuncPtr)])
      externalsFunc.setType(self.IOR.getExternalType(type))
      externalsFunc.setInitializer(externalsInit)
      mainBody.addDeclarations([self.Cxx.getDecl(externalsFunc)])
      externalsInd = self.Cxx.getIndirection(externalsVar)
      externalsCall = self.Cxx.getFunctionCall(self.Cxx.getGroup(externalsInd))
    mainBody.addDeclarations([self.Cxx.getDeclaration(extFuncVar, self.IOR.getExternalType(type), self.Cxx.getInitializer(None, externalsCall))])
    arrayDimen = self.Cxx.getFunctionCall(self.Cxx.getGroup(self.Cxx.getIndirection(self.Cxx.getStructRef(extFuncVar, 'getDimen'))), [arrayVar, null])
    dimenDecl = self.Cxx.getDeclaration(dimenVar, self.typeMap['int'], self.Cxx.getInitializer(None, arrayDimen))
    mainBody.addDeclarations([dimenDecl])
    indLowerVar = self.Cxx.getVar('indLower')
    indUpperVar = self.Cxx.getVar('indUpper')
    indStrideVar = self.Cxx.getVar('indStride')
    mainBody.addDeclarations([self.Cxx.getArray(indLowerVar, self.IOR.typeMap['int'], 1, self.Cxx.getInitializer(None, self.Cxx.getInteger(0), isList = 1))])
    mainBody.addDeclarations([self.Cxx.getArray(indUpperVar, self.IOR.typeMap['int'], 1, self.Cxx.getInitializer(None, self.Cxx.getInteger(0), isList = 1))])
    mainBody.addDeclarations([self.Cxx.getArray(indStrideVar, self.IOR.typeMap['int'], 1, self.Cxx.getInitializer(None, self.Cxx.getInteger(0), isList = 1))])
    safeDimen = ASE.Compiler.Cxx.If.If()
    safeDimen.setBranch(dimenVar)
    safeDimen.setChildren([dimenVar, self.Cxx.getInteger(1)])
    safeDimen.setTernary(1)
    lowerMalloc = self.Cxx.getFunctionCall('malloc', [self.Cxx.getMultiplication(self.Cxx.getSizeof(self.IOR.typeMap['int']), self.Cxx.getGroup(safeDimen))])
    lowerInit = self.Cxx.getInitializer(None, [lowerMalloc, indLowerVar, indUpperVar, indStrideVar, self.Cxx.getInteger(0), self.Cxx.getInteger(1)])
    mainBody.addDeclarations([self.Cxx.getDeclaration(lowerVar, self.IOR.getType(intArray).getChildren()[0], lowerInit)])
    mainBody.addDeclarations([self.Cxx.getDeclaration(upperVar, self.IOR.getType(intArray).getChildren()[0], lowerInit)])
    mainBody.addDeclarations([self.Cxx.getDeclaration(numElemVar, self.IOR.getType(intArray).getChildren()[0], lowerInit)])
    mainBody.addDeclarations([self.Cxx.getDeclaration(iVar, self.typeMap['int'])])
    lowerFirst = self.Cxx.getStructRef(lowerVar, 'd_firstElement', 0)
    lowerIndex = self.Cxx.getArrayRef(lowerFirst, iVar)
    lowerCall = self.Cxx.getFunctionCall(self.Cxx.getGroup(self.Cxx.getIndirection(self.Cxx.getStructRef(extFuncVar, 'getLower'))), [arrayVar, iVar, null])
    upperFirst = self.Cxx.getStructRef(upperVar, 'd_firstElement', 0)
    upperIndex = self.Cxx.getArrayRef(upperFirst, iVar)
    upperCall = self.Cxx.getFunctionCall(self.Cxx.getGroup(self.Cxx.getIndirection(self.Cxx.getStructRef(extFuncVar, 'getUpper'))), [arrayVar, iVar, null])
    numElemFirst = self.Cxx.getStructRef(numElemVar, 'd_firstElement', 0)
    numElemIndex = self.Cxx.getArrayRef(numElemFirst, iVar)
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.addChildren(map(self.Cxx.getExpStmt, [self.Cxx.getAssignment(lowerIndex, lowerCall), self.Cxx.getAssignment(upperIndex, upperCall)]))
    compound.addChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(numElemIndex, self.Cxx.getAddition(self.Cxx.getSubtraction(upperIndex, lowerIndex), self.Cxx.getInteger(1))))])
    setDims = self.Cxx.getSimpleLoop(iVar, 0, dimenVar)
    setDims.setChildren([compound])
    mainBody.addChildren([setDims])
    if borrow:
      pyArrayCreate = self.Cxx.getFunctionCall('PyArray_FromDimsAndData', [dimenVar, numElemFirst, self.getNumericTypeCode(type), self.Cxx.getStructRef(arrayVar, 'd_firstElement')])
      mainBody.addChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(returnVar, self.Cxx.castToType(pyArrayCreate, self.arrayType)))])
    else:
      pyArrayCreate = self.Cxx.getFunctionCall('PyArray_FromDims', [dimenVar, numElemFirst, self.getNumericTypeCode(type)])
      mainBody.addChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(returnVar, self.Cxx.castToType(pyArrayCreate, self.arrayType)))])
      compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
      sizeVar = self.Cxx.getVar('size')
      dataVar = self.Cxx.getVar('data')
      compound.addDeclarations([self.Cxx.getDeclaration(sizeVar, self.typeMap['int'], self.Cxx.getInitializer(None, self.Cxx.getInteger(1)))])
      compound.addDeclarations([self.Cxx.getDeclaration(dataVar, self.Cxx.typeMap['char pointer'], self.Cxx.getInitializer(None, self.Cxx.getStructRef(returnVar, 'data')))])
      multSize = ASE.Compiler.Cxx.MultiplicationAssignment.MultiplicationAssignment()
      multSize.setChildren([sizeVar, numElemIndex])
      cmpd = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
      cmpd.setChildren([self.Cxx.getExpStmt(multSize)])
      calcSize = self.Cxx.getSimpleLoop(iVar, 0, dimenVar)
      calcSize.setChildren([cmpd])
      compound.addChildren([calcSize])
      sizeLoopBody = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
      elemVar = self.Cxx.getVar('elem')
      getElement = self.Cxx.getFunctionCall(self.Cxx.getGroup(self.Cxx.getIndirection(self.Cxx.getStructRef(extFuncVar, 'getElement'))), [arrayVar, self.Cxx.getAddress(lowerVar), null])
      sizeLoopBody.setDeclarations([self.Cxx.getDeclaration(elemVar, self.IOR.getType(type), self.Cxx.getInitializer(None, getElement))])
      if type.getBaseType():
        typeName = type.getIdentifier()
        if typeName == 'string':
          setData = self.Cxx.getAssignment(self.Cxx.getIndirection(self.Cxx.castToType(dataVar, self.objType, 1)), self.Cxx.getFunctionCall('PyString_FromString', [elemVar]))
          freeElem = self.Cxx.getFunctionCall('free', [self.Cxx.castToNamedType(elemVar, 'char', 1)])
          trueBranch = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()        
          trueBranch.addChildren([self.Cxx.getExpStmt(setData), self.Cxx.getExpStmt(freeElem)])
          setData = self.Cxx.getAssignment(self.Cxx.getIndirection(self.Cxx.castToType(dataVar, self.objType, 1)), self.Cxx.getVar('Py_None'))
          falseBranch = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()        
          falseBranch.addChildren([self.Cxx.getExpStmt(self.Cxx.getFunctionCall('Py_INCREF', ['Py_None'])), self.Cxx.getExpStmt(setData)])
          checkElem = ASE.Compiler.Cxx.If.If()
          checkElem.setBranch(elemVar)
          checkElem.setChildren([trueBranch, falseBranch])
          sizeLoopBody.addChildren([checkElem])
        elif typeName == 'opaque':
          dataInd = self.Cxx.getIndirection(self.Cxx.castToType(dataVar, self.objType, 1))
          sizeLoopBody.addChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(dataInd, self.Cxx.getFunctionCall('PyCObject_FromVoidPtr', [elemVar, null])))])
        elif typeName in ['dcomplex', 'fcomplex']:
          self.setValue_dcomplex(elemVar, self.Cxx.getIndirection(self.Cxx.castToType(dataVar, self.IOR.getType(type), 1)), sizeLoopBody, setIOR = 0, pythonImagName = 'imaginary')
        else:
          sizeLoopBody.addChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(self.Cxx.getIndirection(self.Cxx.castToType(dataVar, self.getType(type), 1)), elemVar))])
      elif type.isInstanceOf('ASE.Compiler.SIDL.Interface'):
        increfFunc = self.Cxx.getIndirection(self.Cxx.getGroup(self.Cxx.getStructRef(self.Cxx.getStructRef(elemVar, 'd_epv'), 'addReference')))
        sizeLoopBody.addChildren([self.Cxx.getExpStmt(self.Cxx.getFunctionCall(self.Cxx.getGroup(increfFunc), [self.IOR.getTrueIOR(elemVar), null]))])
        dataInd = self.Cxx.getIndirection(self.Cxx.castToType(dataVar, self.objType, 1))
        sizeLoopBody.addChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(dataInd, self.Cxx.getFunctionCall(self.getWrapIOR(type).getChildren()[0], [elemVar])))])
        gotoError = ASE.Compiler.Cxx.Goto.Goto()
        gotoError.setIdentifier('error')
        cmpd = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
        cmpd.setChildren([self.Cxx.getExpStmt(self.Cxx.getFunctionCall('Py_DECREF', [returnVar])), self.Cxx.getExpStmt(self.Cxx.getAssignment(returnVar, null)), gotoError])
        checkData   = ASE.Compiler.Cxx.If.If()
        checkData.setBranch(self.Cxx.getNegation(self.Cxx.getGroup(dataInd)))
        checkData.setChildren([cmpd])
        sizeLoopBody.addChildren([checkData])
      strideIndex = self.Cxx.getArrayRef(self.Cxx.getStructRef(returnVar, 'strides'), iVar)
      incLowerIndex = ASE.Compiler.Cxx.Increment.Increment()
      incLowerIndex.setChildren([self.Cxx.getGroup(lowerIndex)])
      cmpLower = ASE.Compiler.Cxx.LessThanOrEqual.LessThanOrEqual()
      cmpLower.setChildren([incLowerIndex, upperIndex])
      incData = ASE.Compiler.Cxx.AdditionAssignment.AdditionAssignment()
      incData.setChildren([dataVar, strideIndex])
      trueBranch = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
      trueBranch.setChildren([ASE.Compiler.Cxx.Break.Break()])
      decData = ASE.Compiler.Cxx.SubtractionAssignment.SubtractionAssignment()
      decData.setChildren([dataVar, self.Cxx.getMultiplication(strideIndex, numElemIndex)])
      decLower = ASE.Compiler.Cxx.SubtractionAssignment.SubtractionAssignment()
      decLower.setChildren([lowerIndex, numElemIndex])
      falseBranch = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
      falseBranch.setChildren([self.Cxx.getExpStmt(decData), self.Cxx.getExpStmt(decLower)])
      checkLower = ASE.Compiler.Cxx.If.If()
      checkLower.setBranch(cmpLower)
      checkLower.setChildren([trueBranch, falseBranch])
      cmpd = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
      cmpd.setChildren([self.Cxx.getExpStmt(incData), checkLower])
      incLower = self.Cxx.getSimpleLoop(iVar, 0, dimenVar)
      incLower.setChildren([cmpd])
      sizeLoopBody.addChildren([incLower])
      decSize = ASE.Compiler.Cxx.Increment.Increment()
      decSize.setChildren([sizeVar])
      decSize.setDecrement(1)
      decSize.setPostfix(1)
      sizeLoop = ASE.Compiler.Cxx.While.While()
      sizeLoop.setBranch(decSize)
      sizeLoop.setChildren([sizeLoopBody])
      compound.addChildren([sizeLoop])
      checkReturn = ASE.Compiler.Cxx.If.If()
      checkReturn.setBranch(returnVar)
      checkReturn.setChildren([compound])
      mainBody.addChildren([checkReturn])
    freeLower = self.Cxx.getExpStmt(self.Cxx.getFunctionCall('free', [lowerFirst]))
    freeLower.setIdentifier('error')
    mainBody.addChildren([freeLower])
    mainBody.addChildren([self.Cxx.getExpStmt(self.Cxx.getFunctionCall('free', [upperFirst])), self.Cxx.getExpStmt(self.Cxx.getFunctionCall('free', [numElemFirst]))])
    checkArray = ASE.Compiler.Cxx.If.If()
    checkArray.setBranch(arrayVar)
    checkArray.setChildren([mainBody])
    body.addChildren([checkArray, self.Cxx.getReturn(self.Cxx.castToType(returnVar, self.objType))])

    wrapName = ASE.Compiler.Cxx.Declarator.Declarator()
    if borrow:
      wrapName.setIdentifier(name.replace('.', '_')+'__wrap_array_borrow')
    else:
      wrapName.setIdentifier(name.replace('.', '_')+'__wrap_array_copy')
    wrapFunc = ASE.Compiler.Cxx.Function.Function()
    wrapFunc.setChildren([wrapName])
    wrapFunc.setType(self.objType)
    wrapFunc.setStatic(1)
    wrapFunc.setParameters([self.Cxx.getParameter(arrayVar.getIdentifier(), self.IOR.getType(arrayType))])
    wrapFunc.setBody(body)
    self.wrapArray[(type, borrow)] = wrapFunc
    return wrapFunc
  
  def getExtractIterator(self, interface, moduleInterface = None):
    '''
    Generate the function which wraps a Python iterator in the appropriate IOR structure
    '''
    import ASE.Compiler.SIDL.Iterator
    import ASE.Compiler.Cxx.Cast
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.Pointer
    if interface in self.extractIterator:
      if moduleInterface is None:
        if isinstance(self.extractIterator[interface], dict):
          return self.extractIterator[interface].values()[0]
        else:
          return self.extractIterator[interface]
      elif moduleInterface in self.extractIterator[interface]:
        return self.extractIterator[interface][moduleInterface]
    body = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    name = interface.getFullIdentifier()
    objVar = self.Cxx.getVar('obj')
    iorVar = self.Cxx.getVar('ior')
    epvVar = self.Cxx.getVar('_epv')
    null = self.Cxx.getNull()
    iterType = ASE.Compiler.SIDL.Iterator.Iterator()
    iterType.setType(interface)
    extType = self.IOR.getExternalType(interface)
    extFuncType = ASE.Compiler.Cxx.Function.Function()
    extFuncType.setChildren([self.Cxx.getDeclGroup(ASE.Compiler.Cxx.Pointer.Pointer())])
    extFuncType.setType(extType)
    extFunc = self.Cxx.getStructRef(self.Cxx.getFunctionCall(self.Cxx.getStructRef(self.Cxx.getFunctionCall('ASE_Loader__externals'), 'getStaticEPV'), [null]), 'getClassMethodTable')
    extCast = ASE.Compiler.Cxx.Cast.Cast()
    extCast.setTypeName(extFuncType)
    extCast.setChildren([self.Cxx.getFunctionCall(extFunc, [self.Cxx.getString(name), null])])
    epvInit = self.Cxx.getInitializer(None, self.Cxx.getFunctionCall(self.Cxx.getGroup(extCast)))
    body.addDeclarations([self.Cxx.getDeclaration(epvVar, extType, epvInit)])
    body.addChildren([self.Cxx.getExpStmt(self.Cxx.getFunctionCall('Py_INCREF', [objVar]))])
    iorObj = self.Cxx.getGroup(self.Cxx.getIndirection(iorVar))
    iterNextFunc = interface.getFullIdentifier().replace('.', '_')+'_iterNext'
    createCall = self.Cxx.getFunctionCall(self.Cxx.getGroup(self.Cxx.getIndirection(self.Cxx.getGroup(self.Cxx.getStructRef(epvVar, 'createIterator')))), [objVar, iterNextFunc, null])
    body.addChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(self.Cxx.getIndirection(iorVar), createCall))])
    returnTrue = self.Cxx.getReturn(self.Cxx.getInteger(1))
    body.addChildren([self.getErrorCheck('PyExc_TypeError', 'Could not create ' + name + ' iterator', iorObj, self.Cxx.getInteger(0)), returnTrue])
    iorPtrType = ASE.Compiler.Cxx.Pointer.Pointer()
    iorPtrType.setChildren([self.IOR.getType(iterType)])
    extractName = ASE.Compiler.Cxx.Declarator.Declarator()
    extractName.setIdentifier(name.replace('.', '_') + '__extract_iterator')
    extractFunc = ASE.Compiler.Cxx.Function.Function()
    extractFunc.setChildren([extractName])
    extractFunc.setType(self.typeMap['int'])
    extractFunc.setStatic(1)
    extractFunc.setParameters([self.Cxx.getParameter(objVar.getIdentifier(), self.objType), self.Cxx.getParameter(iorVar.getIdentifier(), iorPtrType)])
    extractFunc.setBody(body)
    if not interface in self.extractIterator:
      self.extractIterator[interface] = {}

    self.extractIterator[interface][moduleInterface] = extractFunc
    return extractFunc
  
  def parseArgument_scalar(self, parameter, isArgument):
    if isArgument:
      var = self.Cxx.getVar(parameter.getIdentifier())
    else:
      var = self.Cxx.getAddress(self.Cxx.getVar(parameter.getIdentifier()))
    return [var]
  
  def parseConvert_scalar(self, parameter, body, isArgument):
    if not isArgument:
      body.addDeclarations([self.Cxx.getDeclaration(self.Cxx.getVar(parameter.getIdentifier()), self.IOR.getType(parameter.getType()))])
    return
  
  def buildArgument_scalar(self, parameter, isArgument):
    return [self.getBuildConvertVar(parameter, isArgument)]
  
  def buildConvert_scalar(self, parameter, body, isArgument):
    if not isArgument and not self.SIDL.isIn(parameter):
      body.addDeclarations([self.Cxx.getDeclaration(self.Cxx.getVar(parameter.getIdentifier()), self.IOR.getType(parameter.getType()))])
    return
  
  def parseArgument_localTemp(self, parameter, isArgument):
    return [self.Cxx.getAddress(self.Cxx.getVar('_local_' + parameter.getIdentifier()))]
  
  def parseConvert_localTemp(self, parameter, body, isArgument):
    sidlType = parameter.getType()
    iorVar = self.getParseConvertVar(parameter, isArgument)
    iorType = self.IOR.getType(sidlType)
    pythonVar = self.Cxx.getVar('_local_' + parameter.getIdentifier())
    pythonType = self.getType(sidlType)
    if not isArgument:
      if iorVar.isInstanceOf('ASE.Compiler.Cxx.Variable'):
        body.addDeclarations([self.Cxx.getDeclaration(iorVar, iorType)])
    if not isArgument or not self.SIDL.isIn(parameter):
      body.addDeclarations([self.Cxx.getDeclaration(pythonVar, pythonType)])
    body.addChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(iorVar, pythonVar), 'Convert ' + pythonType.getIdentifier() + ' to ' + iorType.getIdentifier())])
    return
  
  def buildArgument_localTemp(self, parameter, isArgument):
    return [self.Cxx.getVar('_local_' + parameter.getIdentifier())]
  
  def buildConvert_localTemp(self, parameter, body, isArgument):
    sidlType = parameter.getType()
    iorVar = self.getBuildConvertVar(parameter, isArgument)
    iorType = self.IOR.getType(sidlType)
    pythonVar = self.Cxx.getVar('_local_' + parameter.getIdentifier())
    pythonType = self.getType(sidlType)
    if not isArgument and not self.SIDL.isIn(parameter):
      body.addDeclarations([self.Cxx.getDeclaration(self.Cxx.getVar(parameter.getIdentifier()), iorType)])
    if isArgument or not self.SIDL.isIn(parameter):
      body.addDeclarations([self.Cxx.getDeclaration(pythonVar, pythonType)])
    body.addChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(pythonVar, iorVar), 'Convert ' + iorType.getIdentifier() + ' to ' + pythonType.getIdentifier())])
    return
  
  def parseFormat_bool(self, parameter):
    return 'i'
  
  def parseArgument_bool(self, parameter, isArgument):
    return self.parseArgument_scalar(parameter, isArgument)
  
  def parseConvert_bool(self, parameter, body, isArgument):
    return self.parseConvert_scalar(parameter, body, isArgument)
  
  def buildArgument_bool(self, parameter, isArgument):
    return self.buildArgument_scalar(parameter, isArgument)
  
  def buildConvert_bool(self, parameter, body, isArgument):
    return self.buildConvert_scalar(parameter, body, isArgument)
  
  def parseFormat_char(self, parameter):
    return 'c'
  
  def parseArgument_char(self, parameter, isArgument):
    return self.parseArgument_scalar(parameter, isArgument)
  
  def parseConvert_char(self, parameter, body, isArgument):
    return self.parseConvert_scalar(parameter, body, isArgument)
  
  def buildArgument_char(self, parameter, isArgument):
    return self.buildArgument_scalar(parameter, isArgument)
  
  def buildConvert_char(self, parameter, body, isArgument):
    return self.buildConvert_scalar(parameter, body, isArgument)
  
  def parseFormat_dcomplex(self, parameter):
    return 'D'
  
  def setValue_dcomplex(self, iorVar, pythonVar, body, setIOR, pythonImagName = 'imag'):
    iorReal = self.Cxx.getStructRef(self.Cxx.getGroup(iorVar), 'real', 0)
    pythonReal = self.Cxx.getStructRef(self.Cxx.getGroup(pythonVar), 'real', 0)
    iorImag = self.Cxx.getStructRef(self.Cxx.getGroup(iorVar), 'imaginary', 0)
    pythonImag = self.Cxx.getStructRef(self.Cxx.getGroup(pythonVar), pythonImagName, 0)
    if setIOR:
      setReal = self.Cxx.getAssignment(iorReal, pythonReal)
      setImag = self.Cxx.getAssignment(iorImag, pythonImag)
    else:
      setReal = self.Cxx.getAssignment(pythonReal, iorReal)
      setImag = self.Cxx.getAssignment(pythonImag, iorImag)
    body.addChildren([self.Cxx.getExpStmt(setReal), self.Cxx.getExpStmt(setImag)])
    return
  
  def parseArgument_dcomplex(self, parameter, isArgument):
    return self.parseArgument_localTemp(parameter, isArgument)
  
  def parseConvert_dcomplex(self, parameter, body, isArgument):
    iorVar = self.getParseConvertVar(parameter, isArgument)
    pythonVar = self.Cxx.getVar('_local_' + parameter.getIdentifier())
    if not isArgument:
      if iorVar.isInstanceOf('ASE.Compiler.Cxx.Variable'):
        body.addDeclarations([self.Cxx.getDeclaration(iorVar, self.IOR.getType(parameter.getType()))])
    if not isArgument or not self.SIDL.isIn(parameter):
      body.addDeclarations([self.Cxx.getDeclaration(pythonVar, self.getType(parameter.getType()))])
    self.setValue_dcomplex(iorVar, pythonVar, body, setIOR = 1)
    return
  
  def buildArgument_dcomplex(self, parameter, isArgument):
    return [self.Cxx.getAddress(self.Cxx.getVar('_local_' + parameter.getIdentifier()))]
  
  def buildConvert_dcomplex(self, parameter, body, isArgument):
    iorVar = self.getBuildConvertVar(parameter, isArgument)
    pythonVar = self.Cxx.getVar('_local_' + parameter.getIdentifier())
    if not isArgument and not self.SIDL.isIn(parameter):
      body.addDeclarations([self.Cxx.getDeclaration(self.Cxx.getVar(parameter.getIdentifier()), self.IOR.getType(parameter.getType()))])
    if isArgument or not self.SIDL.isIn(parameter):
      body.addDeclarations([self.Cxx.getDeclaration(pythonVar, self.getType(parameter.getType()))])
    self.setValue_dcomplex(iorVar, pythonVar, body, setIOR = 0)
    return
  
  def parseFormat_double(self, parameter):
    return 'd'
  
  def parseArgument_double(self, parameter, isArgument):
    return self.parseArgument_scalar(parameter, isArgument)
  
  def parseConvert_double(self, parameter, body, isArgument):
    return self.parseConvert_scalar(parameter, body, isArgument)
  
  def buildArgument_double(self, parameter, isArgument):
    return self.buildArgument_scalar(parameter, isArgument)
  
  def buildConvert_double(self, parameter, body, isArgument):
    return self.buildConvert_scalar(parameter, body, isArgument)
  
  def parseFormat_fcomplex(self, parameter):
    return 'D'
  
  def parseArgument_fcomplex(self, parameter, isArgument):
    return self.parseArgument_localTemp(parameter, isArgument)
  
  def parseConvert_fcomplex(self, parameter, body, isArgument):
    return self.parseConvert_dcomplex(parameter, body, isArgument)
  
  def buildArgument_fcomplex(self, parameter, isArgument):
    return self.buildArgument_dcomplex(parameter, isArgument)
  
  def buildConvert_fcomplex(self, parameter, body, isArgument):
    return self.buildConvert_dcomplex(parameter, body, isArgument)
  
  def parseFormat_float(self, parameter):
    return 'f'
  
  def parseArgument_float(self, parameter, isArgument):
    return self.parseArgument_scalar(parameter, isArgument)
  
  def parseConvert_float(self, parameter, body, isArgument):
    return self.parseConvert_scalar(parameter, body, isArgument)
  
  def buildArgument_float(self, parameter, isArgument):
    return self.buildArgument_scalar(parameter, isArgument)
  
  def buildConvert_float(self, parameter, body, isArgument):
    return self.buildConvert_scalar(parameter, body, isArgument)
  
  def parseFormat_int(self, parameter):
    return 'l'
  
  def parseArgument_int(self, parameter, isArgument):
    return self.parseArgument_localTemp(parameter, isArgument)
  
  def parseConvert_int(self, parameter, body, isArgument):
    return self.parseConvert_localTemp(parameter, body, isArgument)
  
  def buildArgument_int(self, parameter, isArgument):
    return self.buildArgument_localTemp(parameter, isArgument)
  
  def buildConvert_int(self, parameter, body, isArgument):
    return self.buildConvert_localTemp(parameter, body, isArgument)
  
  def parseFormat_long(self, parameter):
    return 'L'
  
  def parseArgument_long(self, parameter, isArgument):
    return self.parseArgument_localTemp(parameter, isArgument)
  
  def parseConvert_long(self, parameter, body, isArgument):
    return self.parseConvert_localTemp(parameter, body, isArgument)
  
  def buildArgument_long(self, parameter, isArgument):
    return self.buildArgument_localTemp(parameter, isArgument)
  
  def buildConvert_long(self, parameter, body, isArgument):
    return self.buildConvert_localTemp(parameter, body, isArgument)
  
  def parseFormat_opaque(self, parameter):
    return 'O&'
  
  def parseArgument_opaque(self, parameter, isArgument):
    if isArgument:
      var = self.Cxx.getVar(parameter.getIdentifier())
    else:
      var = self.Cxx.getAddress(self.Cxx.getVar(parameter.getIdentifier()))
    return [self.Cxx.castToVoidPointer(self.getExtractOpaque(parameter.getType()).getChildren()[0]), var]
  
  def parseConvert_opaque(self, parameter, body, isArgument):
    return self.parseConvert_scalar(parameter, body, isArgument)
  
  def buildArgument_opaque(self, parameter, isArgument):
    return [self.Cxx.castToVoidPointer(self.getWrapOpaque(parameter.getType()).getChildren()[0]), self.getBuildConvertVar(parameter, isArgument)]
  
  def buildConvert_opaque(self, parameter, body, isArgument):
    return self.buildConvert_scalar(parameter, body, isArgument)
  
  def copyValue_string(self, var, body):
    import ASE.Compiler.Cxx.Addition
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.If
    strLen = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    strLen.setChildren([self.Cxx.getVar('strlen')])
    strLen.setArguments([var])
    strSize = ASE.Compiler.Cxx.Addition.Addition()
    strSize.setChildren([strLen, self.Cxx.getInteger(1)])
    strAlloc = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    strAlloc.setChildren([self.Cxx.getVar('malloc')])
    strAlloc.setArguments([strSize])
    strCopy = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    strCopy.setChildren([self.Cxx.getVar('strcpy')])
    strCopy.setArguments([strAlloc, var])
    strCheck = ASE.Compiler.Cxx.If.If()
    strCheck.setBranch(var)
    strCheck.setChildren([strCopy, self.Cxx.getNull()])
    strCheck.setTernary(1)
    strSet = ASE.Compiler.Cxx.Assignment.Assignment()
    strSet.setChildren([var, strCheck])
    body.addChildren([self.Cxx.getExpStmt(strSet, 'Copying the Python string to IOR controlled memory')])
    return
  
  def parseFormat_string(self, parameter):
    return 'z'
  
  def parseArgument_string(self, parameter, isArgument):
    return self.parseArgument_scalar(parameter, isArgument)
  
  def parseConvert_string(self, parameter, body, isArgument):
    if not isArgument:
      body.addDeclarations([self.Cxx.getDeclaration(self.Cxx.getVar(parameter.getIdentifier()), self.IOR.getType(parameter.getType()))])
    if not isArgument or self.SIDL.isOut(parameter):
      self.copyValue_string(self.getParseConvertVar(parameter, isArgument), body)
    return
  
  def buildArgument_string(self, parameter, isArgument):
    return self.buildArgument_scalar(parameter, isArgument)
  
  def buildConvert_string(self, parameter, body, isArgument):
    return self.buildConvert_scalar(parameter, body, isArgument)
  
  def buildCleanup_string(self, parameter, body, isArgument):
    '''
    Should free string memory after a build call, unless the original string can be referenced
    '''
    if not isArgument:
      import ASE.Compiler.Cxx.FunctionCall
      var = self.getBuildConvertVar(parameter, isArgument)
      freeCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      freeCall.setChildren([self.Cxx.getVar('free')])
      freeCall.setArguments([self.Cxx.castToVoidPointer(var)])
      body.addChildren([self.Cxx.getExpStmt(freeCall, 'Freeing IOR memory since Python has made a copy of the string')])
    return
  
  def parseFormat_function(self, parameter):
    return 'O'
  
  def parseArgument_function(self, parameter, isArgument):
    return self.parseArgument_scalar(parameter, isArgument)
  
  def parseConvert_function(self, parameter, body, isArgument):
    return self.parseConvert_scalar(parameter, body, isArgument)
  
  def buildArgument_function(self, parameter, isArgument):
    return self.buildArgument_scalar(parameter, isArgument)
  
  def buildConvert_function(self, parameter, body, isArgument):
    return self.buildConvert_scalar(parameter, body, isArgument)
  
  def buildCleanup_function(self, parameter, body, isArgument):
    return
  
  def parseFormatEnumeration(self, parameter):
    return self.parseFormat_int(parameter)
  
  def parseArgumentEnumeration(self, parameter, isArgument):
    return self.parseArgument_localTemp(parameter, isArgument)
  
  def parseConvertEnumeration(self, parameter, body, isArgument):
    return self.parseConvert_localTemp(parameter, body, isArgument)
  
  def buildArgumentEnumeration(self, parameter, isArgument):
    return self.buildArgument_localTemp(parameter, isArgument)
  
  def buildConvertEnumeration(self, parameter, body, isArgument):
    return self.buildConvert_localTemp(parameter, body, isArgument)
  
  def buildCleanupEnumeration(self, parameter, body, isArgument):
    return
  
  def parseFormatArray(self, parameter):
    return self.parseFormat_opaque(parameter)
  
  def parseArgumentArray(self, parameter, isArgument):
    import ASE.Compiler.SIDL.Array
    entryType = ASE.Compiler.SIDL.Array.Array(parameter.getType()).getType()
    if isArgument:
      var = self.Cxx.getVar(parameter.getIdentifier())
    else:
      var = self.Cxx.getAddress(self.Cxx.getVar(parameter.getIdentifier()))
    return [self.Cxx.castToVoidPointer(self.getExtractArray(entryType, not self.SIDL.isOut(parameter) and self.canBorrowArray(entryType)).getChildren()[0]), var]
  
  def parseConvertArray(self, parameter, body, isArgument):
    return self.parseConvert_scalar(parameter, body, isArgument)
  
  def buildArgumentArray(self, parameter, isArgument):
    import ASE.Compiler.SIDL.Array
    entryType = ASE.Compiler.SIDL.Array.Array(parameter.getType()).getType()
    return [self.Cxx.castToVoidPointer(self.getWrapArray(entryType, self.canBorrowArray(entryType)).getChildren()[0]), self.getBuildConvertVar(parameter, isArgument)]
  
  def buildConvertArray(self, parameter, body, isArgument):
    return self.buildConvert_scalar(parameter, body, isArgument)
  
  def buildCleanupArray(self, parameter, body, isArgument):
    if self.SIDL.isOut(parameter):
      self.getCleanupArray(parameter, body)
    return
  
  def parseFormatIterator(self, parameter):
    return self.parseFormat_opaque(parameter)
  
  def parseArgumentIterator(self, parameter, isArgument):
    import ASE.Compiler.SIDL.Iterator
    entryType = ASE.Compiler.SIDL.Iterator.Iterator(parameter.getType()).getType()
    if isArgument:
      var = self.Cxx.getVar(parameter.getIdentifier())
    else:
      var = self.Cxx.getAddress(self.Cxx.getVar(parameter.getIdentifier()))
    return [self.Cxx.castToVoidPointer(self.getExtractIterator(entryType).getChildren()[0]), var]
  
  def parseConvertIterator(self, parameter, body, isArgument):
    return self.parseConvert_scalar(parameter, body, isArgument)
  
  def buildArgumentIterator(self, parameter, isArgument):
    import ASE.Compiler.SIDL.Iterator
    entryType = ASE.Compiler.SIDL.Iterator.Iterator(parameter.getType()).getType()
    return [self.Cxx.castToVoidPointer(self.getWrapIOR(entryType, isWeak = 1).getChildren()[0]), self.getBuildConvertVar(parameter, isArgument)]
  
  def buildConvertIterator(self, parameter, body, isArgument):
    return self.buildConvert_scalar(parameter, body, isArgument)
  
  def buildCleanupIterator(self, parameter, body, isArgument):
    return
  
  def parseFormatInterface(self, parameter):
    return self.parseFormat_opaque(parameter)
  
  def parseArgumentInterface(self, parameter, isArgument):
    if isArgument:
      var = self.Cxx.getVar(parameter.getIdentifier())
    else:
      var = self.Cxx.getAddress(self.Cxx.getVar(parameter.getIdentifier()))
    return [self.Cxx.castToVoidPointer(self.getExtractIOR(parameter.getType()).getChildren()[0]), var]
  
  def parseConvertInterface(self, parameter, body, isArgument):
    return self.parseConvert_scalar(parameter, body, isArgument)
  
  def buildArgumentInterface(self, parameter, isArgument, isWeak = 0):
    return [self.Cxx.castToVoidPointer(self.getWrapIOR(parameter.getType(), isWeak = isWeak).getChildren()[0]), self.getBuildConvertVar(parameter, isArgument)]
  
  def buildConvertInterface(self, parameter, body, isArgument):
    return self.buildConvert_scalar(parameter, body, isArgument)
  
  def buildCleanupInterface(self, parameter, body, isArgument):
    if self.SIDL.isOut(parameter):
      self.getCleanupInterface(parameter, body)
    return
  
  def getParseFormat(self, parameter):
    '''
    Get the Python format string corresponding to a SIDL type
    '''
    type = parameter.getType()
    funcName = 'parseFormat_' + str(type.getIdentifier())
    if hasattr(self, funcName):
      return getattr(self, funcName)(parameter)
    elif type.isInstanceOf('ASE.Compiler.SIDL.Enumeration'):
      return self.parseFormatEnumeration(parameter)
    elif type.isInstanceOf('ASE.Compiler.SIDL.Array'):
      return self.parseFormatArray(parameter)
    elif type.isInstanceOf('ASE.Compiler.SIDL.Iterator'):
      return self.parseFormatIterator(parameter)
    elif type.isInstanceOf('ASE.Compiler.SIDL.Interface'):
      return self.parseFormatInterface(parameter)
    return ''
  
  def getParseArguments(self, parameter, isArgument):
    '''
    Return a list of arguments to be passed to PyArg_Parse* functions for the given parameter
    '''
    type = parameter.getType()
    funcName = 'parseArgument_' + str(type.getIdentifier())
    if hasattr(self, funcName):
      params = getattr(self, funcName)(parameter, isArgument)
    elif type.isInstanceOf('ASE.Compiler.SIDL.Enumeration'):
      params = self.parseArgumentEnumeration(parameter, isArgument)
    elif type.isInstanceOf('ASE.Compiler.SIDL.Array'):
      params = self.parseArgumentArray(parameter, isArgument)
    elif type.isInstanceOf('ASE.Compiler.SIDL.Iterator'):
      params = self.parseArgumentIterator(parameter, isArgument)
    elif type.isInstanceOf('ASE.Compiler.SIDL.Interface'):
      params = self.parseArgumentInterface(parameter, isArgument)
    else:
      params = []
    return params
  
  def getParseConvertVar(self, parameter, isArgument):
    import ASE.Compiler.ParameterAccessMode
    import ASE.Compiler.Cxx.Indirection
    if not isArgument or parameter.getAccessMode() == ASE.Compiler.ParameterAccessMode.RETURN:
      var = self.Cxx.getVar(parameter.getIdentifier())
    else:
      var = ASE.Compiler.Cxx.Indirection.Indirection()
      var.setChildren([self.Cxx.getVar(parameter.getIdentifier())])
    return var
  
  def getConvertParseArgument(self, parameter, body, isArgument):
    '''
    Here we convert from local Python parameters to IOR parameters
       - Declaration of both parameters is also done here
    '''
    type = parameter.getType()
    funcName = 'parseConvert_' + str(type.getIdentifier())
    if hasattr(self, funcName):
      getattr(self, funcName)(parameter, body, isArgument)
    elif type.isInstanceOf('ASE.Compiler.SIDL.Enumeration'):
      self.parseConvertEnumeration(parameter, body, isArgument)
    elif type.isInstanceOf('ASE.Compiler.SIDL.Array'):
      self.parseConvertArray(parameter, body, isArgument)
    elif type.isInstanceOf('ASE.Compiler.SIDL.Iterator'):
      self.parseConvertIterator(parameter, body, isArgument)
    elif type.isInstanceOf('ASE.Compiler.SIDL.Interface'):
      self.parseConvertInterface(parameter, body, isArgument)
    return
  
  def getBuildArguments(self, parameter, isArgument):
    '''
    Convert IOR out parameters to types needed by Py_BuildValue() and construct the parameter list
       - Local variables (_local_foo) are of the type needed by Python
    '''
    type = parameter.getType()
    funcName = 'buildArgument_' + str(type.getIdentifier())
    if hasattr(self, funcName):
      params = getattr(self, funcName)(parameter, isArgument)
    elif type.isInstanceOf('ASE.Compiler.SIDL.Enumeration'):
      params = self.buildArgumentEnumeration(parameter, isArgument)
    elif type.isInstanceOf('ASE.Compiler.SIDL.Array'):
      params = self.buildArgumentArray(parameter, isArgument)
    elif type.isInstanceOf('ASE.Compiler.SIDL.Iterator'):
      params = self.buildArgumentIterator(parameter, isArgument)
    elif type.isInstanceOf('ASE.Compiler.SIDL.Interface'):
      params = self.buildArgumentInterface(parameter, isArgument)
    else:
      params = []
    return params
  
  def getBuildConvertVar(self, parameter, isArgument):
    if isArgument and self.SIDL.isOut(parameter):
      var = self.Cxx.getIndirection(self.Cxx.getVar(parameter.getIdentifier()))
    else:
      var = self.Cxx.getVar(parameter.getIdentifier())
    return var
  
  def getConvertBuildParameter(self, parameter, body, isArgument):
    type = parameter.getType()
    funcName = 'buildConvert_' + str(type.getIdentifier())
    if hasattr(self, funcName):
      getattr(self, funcName)(parameter, body, isArgument)
    elif type.isInstanceOf('ASE.Compiler.SIDL.Enumeration'):
      self.buildConvertEnumeration(parameter, body, isArgument)
    elif type.isInstanceOf('ASE.Compiler.SIDL.Array'):
      self.buildConvertArray(parameter, body, isArgument)
    elif type.isInstanceOf('ASE.Compiler.SIDL.Iterator'):
      self.buildConvertIterator(parameter, body, isArgument)
    elif type.isInstanceOf('ASE.Compiler.SIDL.Interface'):
      self.buildConvertInterface(parameter, body, isArgument)
    return
  
  def getCleanupBuildParameter(self, parameter, body, isArgument):
    paramType = parameter.getType()
    funcName = 'buildCleanup_' + str(paramType.getIdentifier())
    if hasattr(self, funcName):
      getattr(self, funcName)(parameter, body, isArgument)
    elif paramType.isInstanceOf('ASE.Compiler.SIDL.Enumeration'):
      self.buildCleanupEnumeration(parameter, body, isArgument)
    elif paramType.isInstanceOf('ASE.Compiler.SIDL.Array'):
      self.buildCleanupArray(parameter, body, isArgument)
    elif paramType.isInstanceOf('ASE.Compiler.SIDL.Iterator'):
      self.buildCleanupIterator(parameter, body, isArgument)
    elif paramType.isInstanceOf('ASE.Compiler.SIDL.Interface'):
      self.buildCleanupInterface(parameter, body, isArgument)
    return

  def getCleanupArray(self, parameter, body, isArgument = 0):
    import ASE.Compiler.SIDL.Array
    entryType = ASE.Compiler.SIDL.Array.Array(parameter.getType()).getType()
    if entryType.isInstanceOf('ASE.Compiler.SIDL.Interface') and not entryType.getFullIdentifier() in ['ASE.BaseInterface', 'ASE.BaseClass', 'ASE.DLL', 'ASE.Loader']:
      import ASE.Compiler.Cxx.Cast
      import ASE.Compiler.Cxx.Function
      import ASE.Compiler.Cxx.Pointer
      staticEPVCall = self.Cxx.getFunctionCall(self.Cxx.getStructRef(self.Cxx.getFunctionCall('ASE_Loader__externals'), 'getStaticEPV'), [self.Cxx.getNull()])
      objExternalsCall = self.Cxx.getFunctionCall(self.Cxx.getStructRef(staticEPVCall, 'getClassMethodTable'), [self.Cxx.getString(entryType.getFullIdentifier()), self.Cxx.getNull()])
      externalsFuncType = ASE.Compiler.Cxx.Function.Function()
      externalsFuncType.setChildren([self.Cxx.getDeclGroup(ASE.Compiler.Cxx.Pointer.Pointer())])
      externalsFuncType.setType(self.IOR.getExternalType(entryType))
      externalsCast = ASE.Compiler.Cxx.Cast.Cast()
      externalsCast.setChildren([objExternalsCall])
      externalsCast.setTypeName(externalsFuncType)
      objExt = self.Cxx.getFunctionCall(self.Cxx.getGroup(self.Cxx.getIndirection(self.Cxx.getGroup(externalsCast))))
    else:
      objExt = self.Cxx.getFunctionCall(entryType.getFullIdentifier().replace('.', '_') + '__externals')
    destroyFunc = self.Cxx.getIndirection(self.Cxx.getGroup(self.Cxx.getStructRef(objExt, 'destroyArray')))
    body.addChildren([self.Cxx.getExpStmt(self.Cxx.getFunctionCall(self.Cxx.getGroup(destroyFunc), [parameter.getIdentifier(), self.Cxx.getNull()]))])
    return

  def getCleanupInterface(self, parameter, body, isArgument = 0):
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.If
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([self.IOR.addReference(self.Cxx.getGroup(self.Cxx.castToType(self.Cxx.getVar(parameter.getIdentifier()), self.IOR.getBaseType(parameter.getType()))))])
    checkVar = ASE.Compiler.Cxx.If.If()
    checkVar.setBranch(self.Cxx.getVar(parameter.getIdentifier()))
    checkVar.setChildren([compound])
    body.addChildren([checkVar])
    return
  
  def marshalPythonToCCall(self, method, body):
    '''
    This method
       - Declares keyword arguments and check variables
       - Builds format and parse arguments
       - Adds any conversion code
       - Parses Python arguments
    '''
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.Initializer
    import ASE.Compiler.Cxx.Return
    kwInit = ASE.Compiler.Cxx.Initializer.Initializer()
    kwInit.setChildren([self.Cxx.castToString(a.getIdentifier()) for a in method.getInParameters()] + [self.Cxx.getVar('NULL')])
    kwInit.setList(1)
    body.addDeclarations([self.Cxx.getArray('_kwlist', self.Cxx.typeMap['char pointer'], None, kwInit, isStatic = 1)])
    optionalArgs = 0
    format = ''
    localArgs = []
    for parameter in method.getInParameters():
      if not optionalArgs:
        if not parameter.getRequired():
          format += '|'
          optionalArgs = 1
      elif parameter.getRequired():
        self.raiseException('Invalid required argument ' + parameter.getIdentifier() + ' after optional arguments')
      format += self.getParseFormat(parameter)
      localArgs.extend(self.getParseArguments(parameter, isArgument = 0))
    checkVar = self.Cxx.getVar('_okay')
    parseCall = self.Cxx.getFunctionCall('PyArg_ParseTupleAndKeywords', ['_args', '_kwdict', self.Cxx.castToString(format), '_kwlist'] + localArgs)
    body.addDeclarations([self.Cxx.getDeclaration(checkVar, self.typeMap['int'])])
    body.addChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(checkVar, parseCall), 'Parse arguments')])
    for parameter in method.getInParameters():
      self.getConvertParseArgument(parameter, body, isArgument = 0)
    checkExp = ASE.Compiler.Cxx.Negation.Negation()
    checkExp.setChildren([checkVar])
    returnVal = ASE.Compiler.Cxx.Return.Return()
    returnVal.setChildren([self.Cxx.getVar('_pyReturn')])
    returnVal.setComments([self.Cxx.getComment('Let exceptions propagate')])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([returnVal])
    check = ASE.Compiler.Cxx.If.If()
    check.setBranch(checkExp)
    check.setChildren([compound])
    body.addChildren([check])
    return
  
  def marshalPythonToCReturn(self, klass, method, body):
    '''
    This method:
       - Declares the Python and IOR return values
       - Declares local arguments, builds format, and adds any conversion code
       - Parses and converts the local Python return value
       - Cleans up arguments
       It also contains special purpose destruction code
    '''
    import ASE.Compiler.ParameterAccessMode
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.Inequality
    import ASE.Compiler.Cxx.LessThanOrEqual
    import ASE.Compiler.Cxx.LogicalAnd
    import ASE.Compiler.Cxx.Return
    pyReturnVar = self.Cxx.getVar('_pyReturn')
    body.addDeclarations([self.Cxx.getDeclaration(pyReturnVar, self.objType)])
    format = ''
    localArgs = []
    for parameter in method.getOutParameters():
      format += self.getParseFormat(parameter)
      localArgs.extend(self.getParseArguments(parameter, isArgument = not parameter.getAccessMode() == ASE.Compiler.ParameterAccessMode.RETURN))
    if len(method.getOutParameters()):
      if len(method.getOutParameters()) == 1:
        tupleVar = self.Cxx.getVar('returnTuple')
        body.addDeclarations([self.Cxx.getDeclaration(tupleVar, self.objType)])
        body.addChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(tupleVar, self.Cxx.getFunctionCall('PyTuple_New', [1])))])
        body.addChildren([self.Cxx.getExpStmt(self.Cxx.getFunctionCall('PyTuple_SetItem', [tupleVar, 0, pyReturnVar]))])
      else:
        tupleVar = pyReturnVar
      checkVar = self.Cxx.getVar('_okay')
      body.addChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(checkVar, self.Cxx.getFunctionCall('PyArg_ParseTuple', [tupleVar, self.Cxx.castToString(format)] + localArgs)), 'Parse arguments')])
      body.addChildren([self.getClearError()])
      body.addChildren([self.IOR.getErrorCheck(klass, method, checkVar, 'Could not parse return value for ' + klass.getFullIdentifier() + '.' + method.getIdentifier())])
    else:
      isNone = ASE.Compiler.Cxx.Equality.Equality()
      isNone.setChildren([pyReturnVar, self.Cxx.getVar('Py_None')])
      notNone = ASE.Compiler.Cxx.Negation.Negation()
      notNone.setChildren([self.Cxx.getGroup(isNone)])
      body.addChildren([self.IOR.getErrorCheck(klass, method, None, 'Invalid return from void method ' + klass.getFullIdentifier() + '.' + method.getIdentifier(), notNone)])
    for parameter in method.getOutParameters():
      self.getConvertParseArgument(parameter, body, isArgument = not parameter.getAccessMode() == ASE.Compiler.ParameterAccessMode.RETURN)
    if len(method.getOutParameters()):
      if len(method.getOutParameters()) == 1:
        body.addChildren([self.Cxx.getExpStmt(self.Cxx.getFunctionCall('Py_DECREF', [tupleVar]), 'Cleanup local return value')])
      else:
        notNone = ASE.Compiler.Cxx.Inequality.Inequality()
        notNone.setChildren([pyReturnVar, self.Cxx.getVar('Py_None')])
        bothChecks = ASE.Compiler.Cxx.LogicalAnd.LogicalAnd()
        bothChecks.setChildren([pyReturnVar, self.Cxx.getGroup(notNone)])
        compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
        compound.setChildren([self.Cxx.getExpStmt(self.Cxx.getFunctionCall('Py_DECREF', [pyReturnVar]))])
        checkReturn = ASE.Compiler.Cxx.If.If()
        checkReturn.setBranch(bothChecks)
        checkReturn.setChildren([compound])
        checkReturn.setComments([self.Cxx.getComment('Cleanup local return value')])
        body.addChildren([checkReturn])
    body.addChildren([self.Cxx.getExpStmt(self.Cxx.getFunctionCall('Py_DECREF', ['_args']), 'Cleanup arguments')])
    for parameter in method.getInParameters():
      self.getCleanupBuildParameter(parameter, body, isArgument = 1)
    if klass.getFullIdentifier() == 'ASE.BaseClass' and method.getFullIdentifier() == 'ASE.BaseInterface.deleteReference':
      refCountCall = self.Cxx.getFunctionCall(self.Cxx.getStructRef(self.Cxx.getStructRef(self.Cxx.getVar('_self'), 'd_epv'), 'getReferenceCount'), ['_self', '_exception'])
      isCountZero = ASE.Compiler.Cxx.LessThanOrEqual.LessThanOrEqual()
      isCountZero.setChildren([refCountCall, self.Cxx.getInteger(0)])
      destroyCall = self.Cxx.getFunctionCall(self.Cxx.getStructRef(self.Cxx.getStructRef(self.Cxx.getVar('_self'), 'd_epv'), 'destroyObject'), ['_self', '_exception'])
      compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
      compound.setChildren([self.Cxx.getExpStmt(destroyCall)])
      checkRefCount = ASE.Compiler.Cxx.If.If()
      checkRefCount.setBranch(isCountZero)
      checkRefCount.setChildren([compound])
      checkRefCount.setComments([self.Cxx.getComment('Cleanup implementation')])
      body.addChildren([checkRefCount, self.getRestoreError()])
    returnStmt = ASE.Compiler.Cxx.Return.Return()
    if not method.getReturnParameter() is None:
      returnStmt.setChildren([self.Cxx.getVar('_return_value')])
    body.addChildren([returnStmt])
    return
  
  def marshalCToPythonCall(self, klass, method, body, isConstructor = 0, ignoreParameters = []):
    '''
    This method
       - Declares local arguments, builds format, and adds any conversion code
       - Converts arguments from IOR to Python
       - Builds Python arguments
    '''
    argsVar = self.Cxx.getVar('_args')
    format = ''
    localArgs = []
    for parameter in method.getInParameters():
      if parameter.getIdentifier() in ignoreParameters:
        continue
      format += self.getParseFormat(parameter)
      localArgs.extend(self.getBuildArguments(parameter, isArgument = 1))
    format = '(' + format + ')'
    for parameter in method.getInParameters():
      if parameter.getIdentifier() in ignoreParameters:
        continue
      paramType = parameter.getType()
      var = self.Cxx.getVar(parameter.getIdentifier())
      self.getConvertBuildParameter(parameter, body, isArgument = 1)
      if not isConstructor and paramType.isInstanceOf('ASE.Compiler.SIDL.Interface'):
        import ASE.Compiler.Cxx.CompoundStatement
        import ASE.Compiler.Cxx.If
        compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
        compound.setChildren([self.IOR.addReference(self.Cxx.getGroup(self.Cxx.castToType(var, self.IOR.getBaseType(paramType))))])
        checkParam = ASE.Compiler.Cxx.If.If()
        checkParam.setBranch(var)
        checkParam.setChildren([compound])
        body.addChildren([checkParam])
    buildCall = self.Cxx.getFunctionCall('Py_BuildValue', [self.Cxx.castToString(format)] + localArgs)
    setArgs = self.Cxx.getAssignment(argsVar, buildCall)
    errorMsg = 'Unable to parse arguments for ' + klass.getFullIdentifier() + '.' + method.getIdentifier()
    body.addDeclarations([self.Cxx.getDeclaration(argsVar, self.objType), self.Cxx.getDeclaration(self.Cxx.getVar('_okay'), self.typeMap['int'])])
    body.addChildren([self.Cxx.getExpStmt(setArgs, 'Pack arguments for dispatch to implementation'), self.IOR.getErrorCheck(klass, method, argsVar, errorMsg)])
    return
  
  def marshalCToPythonReturn(self, method, body):
    '''
    This method:
       - Declares the Python and IOR return values
       - Declares local arguments, builds format, and adds any conversion code
       - Builds the Python return value
       - Cleans up arguments
    '''
    returnVar = self.Cxx.getVar('_pyReturn')
    format = ''
    localArgs = []
    for parameter in method.getOutParameters():
      format += self.getParseFormat(parameter)
      localArgs.extend(self.getBuildArguments(parameter, isArgument = 0))
    for parameter in method.getOutParameters():
      self.getConvertBuildParameter(parameter, body, isArgument = 0)
    if len(method.getOutParameters()):
      if len(method.getOutParameters()) > 1:
        format = '(' + format + ')'
      body.addChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(returnVar, self.Cxx.getFunctionCall('Py_BuildValue', [self.Cxx.castToString(format)] + localArgs)), 'Build return values')])
    else:
      body.addChildren([self.Cxx.getExpStmt(self.Cxx.getAssignment(returnVar, self.Cxx.getVar('Py_None')), 'Build return values'), self.Cxx.getExpStmt(self.Cxx.getFunctionCall('Py_INCREF', [returnVar]))])
    parameters = list(method.getChildren())
    if not method.getReturnParameter() is None:
      parameters.append(method.getReturnParameter())
    for child in parameters:
      import ASE.Compiler.SIDL.Parameter
      parameter = ASE.Compiler.SIDL.Parameter.Parameter(child)
      self.getCleanupBuildParameter(parameter, body, isArgument = 0)
    return
  
  def dispatch(self, method, body):
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.Return
    setReturn = self.Cxx.getAssignment(self.Cxx.getVar('_pyReturn'), self.Cxx.getFunctionCall('PyObject_CallObject', ['_method', '_args']))
    body.addChildren([self.Cxx.getExpStmt(setReturn, 'Dispatch to implementation'), self.Cxx.getExpStmt(self.Cxx.getFunctionCall('Py_DECREF', ['_method']))])
    returnFalse = ASE.Compiler.Cxx.Return.Return()
    if not method.getReturnParameter() is None:
      returnFalse.setChildren([self.Cxx.getVar(method.getReturnParameter().getIdentifier())])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([self.Cxx.getExpStmt(self.Cxx.getFunctionCall('PyErr_Print')), returnFalse])
    negReturn = ASE.Compiler.Cxx.Negation.Negation()
    negReturn.setChildren([self.Cxx.getVar('_pyReturn')])
    checkReturn = ASE.Compiler.Cxx.If.If()
    checkReturn.setBranch(negReturn)
    checkReturn.setChildren([compound])
    body.addChildren([checkReturn])
    return

  def getClientFunction(self, method):
    '''Return an empty Python stub function, implemented in C'''
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.Parameter
    import ASE.Compiler.Cxx.Return
    methodName = ASE.Compiler.Cxx.Declarator.Declarator()
    methodName.setIdentifier(method.getFullIdentifier().replace('.', '_'))
    methodStub = ASE.Compiler.Cxx.Function.Function()
    methodStub.setChildren([methodName])
    methodStub.setStatic(1)
    methodStub.setType(self.objType)
    selfParam = ASE.Compiler.Cxx.Parameter.Parameter()
    if method.getStatic():
      selfParam.setIdentifier('_ignored')
    else:
      selfParam.setIdentifier('_self')
    selfParam.setType(self.objType)
    argsParam = ASE.Compiler.Cxx.Parameter.Parameter()
    argsParam.setIdentifier('_args')
    argsParam.setType(self.objType)
    kwdictParam = ASE.Compiler.Cxx.Parameter.Parameter()
    kwdictParam.setIdentifier('_kwdict')
    kwdictParam.setType(self.objType)
    methodStub.setParameters([selfParam, argsParam, kwdictParam])
    body = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    methodStub.setBody(body)
    returnVar = self.Cxx.getVar('_pyReturn')
    body.addDeclarations([self.Cxx.getDeclaration(returnVar, self.objType)])
    returnVal = ASE.Compiler.Cxx.Return.Return()
    returnVal.setChildren([returnVar])
    body.addChildren([returnVal])
    return methodStub

  def getClientModule(self, interface):
    '''Return an empty Python stub module, implemented in C'''
    import ASE.Compiler.Cxx.Include
    import ASE.Compiler.Cxx.PythonModule
    module = ASE.Compiler.Cxx.PythonModule.PythonModule()
    module.setClass(interface.getFullIdentifier())
    pyInclude = ASE.Compiler.Cxx.Include.Include()
    pyInclude.setIdentifier('<Python.h>')
    numericInclude = ASE.Compiler.Cxx.Include.Include()
    numericInclude.setIdentifier('<Numeric/arrayobject.h>')
    module.addChildren([pyInclude, numericInclude])
    iorIncludes = [self.IOR.getGuardedInclude(interface), self.IOR.getGuardedInclude(self.SIDL.getLoader(interface)), self.IOR.getGuardedInclude(self.SIDL.getBaseException(interface, 1))]
    module.addChildren([inc for inc in iorIncludes if not inc is None])
    return module
