import ASE.Compiler.SIDL.Class
import ASE.Compiler.SIDL.Interface
import ASE.Compiler.SIDL.Package
import ASE.Compiler.SIDL.Type

class SIDL(object):
  def clear(self):
    del self.typeMap
    return

  def getTypeMap(self):
    if not hasattr(self, '_typeMap'):
      self._typeMap = self.createTypeMap()
    return self._typeMap
  def setTypeMap(self, typeMap):
    self._typeMap = typeMap
    return
  def delTypeMap(self):
    if hasattr(self, '_typeMap'):
      del self._typeMap
    return
  typeMap = property(getTypeMap, setTypeMap, delTypeMap, doc = 'A map from typenames to SIDL type objects')
  
  def createTypeMap(self):
    '''
    Notice that array functions take integer array arguments, and thus all types depend on int
    '''
    typeMap = {}

    for typeName in ['int', 'bool', 'char', 'dcomplex', 'double', 'fcomplex', 'float', 'long', 'opaque', 'function', 'string', 'void']:
      sidlType = ASE.Compiler.SIDL.Type.Type()
      sidlType.setIdentifier(typeName)
      sidlType.setBaseType(1)
      if not typeName == 'int':
        sidlType.setDependencies([typeMap['int']])
      typeMap[typeName] = sidlType
    return typeMap
  
  def getLoader(self, interface):
    for child in self.getBaseInterface(interface).getParent().getChildren():
      if child.getIdentifier() == 'Loader':
        return ASE.Compiler.SIDL.Class.Class(child)
    raise RuntimeError('Could not find Loader using ' + interface.getFullIdentifier())

  def getRoot(self, node):
    if node.isInstanceOf('ASE.Compiler.SIDL.Specification'):
      return node
    return self.getRoot(node.getParent())
  
  def getBasePackage(self, interface):
    return ASE.Compiler.SIDL.Package.Package(self.getAncestor(interface, 'ASE.BaseInterface').getParent())
  
  def getBaseParent(self, interface):
    if interface.isInstanceOf('ASE.Compiler.SIDL.Class'):
      return self.getAncestor(interface, 'ASE.BaseClass')
    return self.getBaseInterface(interface)
  
  def getBaseInterface(self, interface):
    return self.getAncestor(interface, 'ASE.BaseInterface')
  
  def getBaseClass(self, interface):
    package = self.getBasePackage(interface)
    for child in package.getChildren():
      if child.getIdentifier() == 'BaseClass':
        return ASE.Compiler.SIDL.Class.Class(child)
    return
  
  def getBaseException(self, exception, force = 0):
    if force:
      exception = self.getASEException(exception)
    return self.getAncestor(exception, 'ASE.BaseException')
  
  def getASEException(self, interface):
    asePackage = self.getBasePackage(interface)
    for child in asePackage.getChildren():
      if child.getIdentifier() == 'ASEException':
        return ASE.Compiler.SIDL.Class.Class(child)
    return
  
  def getAncestor(self, interface, typeName):
    interface = ASE.Compiler.SIDL.Interface.Interface(interface)
    if interface.getFullIdentifier() == typeName:
      return interface
    parents = []
    if interface.isInstanceOf('ASE.Compiler.SIDL.Class'):
      parents.extend(ASE.Compiler.SIDL.Class.Class(interface).getInterfaces())
    parents.extend(interface.getParents())
    for parent in parents:
      base = self.getAncestor(parent, typeName)
      if base:
        return base
    return

  def findType(self, typeName, startTypes):
    import ASE.Compiler.SIDL.TypeFinder
    if not isinstance(startTypes, list):
      startTypes = [startTypes]
    locator = ASE.Compiler.SIDL.TypeFinder.TypeFinder()
    for tree in [self.getRoot(sidlType) for sidlType in startTypes]:
      locator.setName(typeName)
      tree.accept(locator)
      if locator.hasType():
        return locator.getType()
    return

  def getComment(self, comment):
    import ASE.Compiler.SIDL.Comment
    com = ASE.Compiler.SIDL.Comment.Comment()
    com.setComment(comment)
    return com
  
  def isIn(self, parameter):
    import ASE.Compiler.ParameterAccessMode
    mode = parameter.getAccessMode()
    return mode == ASE.Compiler.ParameterAccessMode.IN or mode == ASE.Compiler.ParameterAccessMode.INOUT
  
  def isOut(self, parameter):
    import ASE.Compiler.ParameterAccessMode
    mode = parameter.getAccessMode()
    return mode == ASE.Compiler.ParameterAccessMode.OUT or mode == ASE.Compiler.ParameterAccessMode.INOUT or mode == ASE.Compiler.ParameterAccessMode.RETURN

  def getMode(self, modeName):
    import ASE.Compiler.ParameterAccessMode
    if modeName == 'in':
      return ASE.Compiler.ParameterAccessMode.IN
    elif modeName == 'out':
      return ASE.Compiler.ParameterAccessMode.OUT
    elif modeName == 'inout':
      return ASE.Compiler.ParameterAccessMode.INOUT
    elif modeName == 'return':
      return ASE.Compiler.ParameterAccessMode.RETURN
    raise RuntimeError('Invalid parameter access mode name: '+str(modeName))

  def getParameter(self, name, type, mode = 'in', isRequired = 1):
    import ASE.Compiler.SIDL.Parameter
    param = ASE.Compiler.SIDL.Parameter.Parameter()
    param.setIdentifier(name)
    param.setType(type)
    param.setAccessMode(self.getMode(mode))
    param.setRequired(isRequired)
    return param
